"""
  Simple Wrapper on the Job class to handle EvnDisp Analysis
  for the Prod5b 2020 analysis from container
  https://forge.in2p3.fr/issues/42322
"""

__RCSID__ = "$Id$"

# generic imports
import json
import collections

# DIRAC imports
from DIRAC.Interfaces.API.Job import Job


class EvnDispSingJob(Job):
    """Job extension class for EvnDisp Analysis,
    takes care of running converter and evndisp.
    """

    def __init__(self, cpuTime=432000):
        """Constructor

        Keyword arguments:
        cpuTime -- max cpu time allowed for the job
        """
        Job.__init__(self)
        self.setCPUTime(cpuTime)
        # defaults
        self.setName("Evndisplay_CalibReco")
        self.setType("EvnDispProcessing")
        self.package = "evndisplay"
        self.version = "eventdisplay-cta-dl1-prod5.v06"
        self.compiler = "gcc48_default"
        self.container = True
        self.program_category = "calibimgreco"
        self.prog_name = "evndisp"
        self.configuration_id = 8
        self.output_data_level = 1
        self.base_path = "/vo.cta.in2p3.fr/MC/PROD5b/"
        self.catalogs = json.dumps(["DIRACFileCatalog", "TSCatalog"])
        self.ts_task_id = 0
        self.output_metadata = collections.OrderedDict()
        self.output_file_metadata = collections.OrderedDict()
        self.group_size = 1

    def set_output_metadata(self, tel_sim_md):
        """Set EventDisplay meta data

        Parameters:
        tel_sim_md -- metadata dictionary from the telescope simulation
        """
        # # Set evndisp directory meta data
        self.output_metadata["array_layout"] = tel_sim_md["array_layout"]
        self.output_metadata["site"] = tel_sim_md["site"]
        self.output_metadata["particle"] = tel_sim_md["particle"]
        try:
            phiP = tel_sim_md["phiP"]["="]
        except TypeError:
            phiP = tel_sim_md["phiP"]
        self.output_metadata["phiP"] = phiP
        try:
            thetaP = tel_sim_md["thetaP"]["="]
        except TypeError:
            thetaP = tel_sim_md["thetaP"]
        self.output_metadata["thetaP"] = thetaP
        self.output_metadata["sct"] = tel_sim_md["sct"]
        self.output_metadata[self.program_category + "_prog"] = self.prog_name
        self.output_metadata[self.program_category + "_prog_version"] = self.version
        self.output_metadata["data_level"] = self.output_data_level
        self.output_metadata["configuration_id"] = self.configuration_id
        self.output_metadata["merged"] = 0
        self.output_metadata["MCCampaign"] = self.MCCampaign

    def set_executable_sequence(self, debug=False):
        """Setup job workflow by defining the sequence of all executables
        All parameters shall have been defined before that method is called.
        """
        i_step = 0
        # step 1 -- debug
        if debug:
            ls_step = self.setExecutable("/bin/ls -alhtr", logFile="LS_Init_Log.txt")
            ls_step["Value"]["name"] = "Step%i_LS_Init" % i_step
            ls_step["Value"]["descr_short"] = "list files in working directory"
            i_step += 1

            env_step = self.setExecutable("/bin/env", logFile="Env_Log.txt")
            env_step["Value"]["name"] = "Step%i_Env" % i_step
            env_step["Value"]["descr_short"] = "Dump environment"
            i_step += 1

        # step 2
        sw_step = self.setExecutable(
            "cta-prod-setup-software",
            arguments="-p %s -v %s -a simulations -g %s"
            % (self.package, self.version, self.compiler),
            logFile="SetupSoftware_Log.txt",
        )
        sw_step["Value"]["name"] = "Step%i_SetupSoftware" % i_step
        sw_step["Value"]["descr_short"] = "Setup software"
        i_step += 1

        # step 3 run EventDisplay
        ev_step = self.setExecutable(
            "./dirac_sing_evndisp.sh", logFile="EvnDisp_Log.txt"
        )
        ev_step["Value"]["name"] = "Step%i_EvnDisplay" % i_step
        ev_step["Value"]["descr_short"] = "Run EvnDisplay"
        i_step += 1

        # step 4 set meta data and register Data
        meta_data_json = json.dumps(self.output_metadata)
        file_meta_data_json = json.dumps(self.output_file_metadata)

        meta_data_field = {
            "array_layout": "VARCHAR(128)",
            "site": "VARCHAR(128)",
            "particle": "VARCHAR(128)",
            "phiP": "float",
            "thetaP": "float",
            "sct": "VARCHAR(128)",
            self.program_category + "_prog": "VARCHAR(128)",
            self.program_category + "_prog_version": "VARCHAR(128)",
            "data_level": "int",
            "configuration_id": "int",
            "merged": "int",
        }
        meta_data_field_json = json.dumps(meta_data_field)

        # register Data
        # to be used with job.version = 'eventdisplay-cta-dl1-prod5.v04'
        # data_output_pattern = './Data/*.simtel.DL1.root'
        # to be used with job.version = 'eventdisplay-cta-dl1-prod5.v06'
        data_output_pattern = "./Data/*.simtel.DL1.tar.gz"
        dm_step = self.setExecutable(
            "cta-prod-managedata",
            arguments="'%s' '%s' '%s' %s '%s' %s %s '%s' Data"
            % (
                meta_data_json,
                meta_data_field_json,
                file_meta_data_json,
                self.base_path,
                data_output_pattern,
                self.package,
                self.program_category,
                self.catalogs,
            ),
            logFile="DataManagement_Log.txt",
        )
        dm_step["Value"]["name"] = "Step%s_DataManagement" % i_step
        dm_step["Value"][
            "descr_short"
        ] = "Save data files to SE and register them in DFC"
        i_step += 1

        # step 5 failover step
        if not debug:
            failover_step = self.setExecutable(
                "/bin/ls -l", modulesList=["Script", "FailoverRequest"]
            )
            failover_step["Value"]["name"] = "Step%s_Failover" % i_step
            failover_step["Value"]["descr_short"] = "Tag files as unused if job failed"
            i_step += 1

        # step 6 - debug only
        if debug:
            ls_step = self.setExecutable("/bin/ls -Ralhtr", logFile="LS_End_Log.txt")
            ls_step["Value"]["name"] = "Step%s_LSHOME_End" % i_step
            ls_step["Value"]["descr_short"] = "list files in Home directory"
            i_step += 1



""" DISET request handler base class for the ProvenanceDB
"""

__RCSID__ = "$Id$"
# imports
import json
import six

# from DIRAC
from DIRAC import gLogger, S_OK, S_ERROR
from DIRAC.Core.DISET.RequestHandler import RequestHandler

# from CTADIRAC
from CTADIRAC.DataManagementSystem.DB.ProvenanceDB import ProvenanceDB


class ProvenanceManagerHandler(RequestHandler):

  """
  .. class:: ProvenanceManagerHandler
  ProvenanceDB interface in the DISET framework.
  """
  __provenanceDB = None

  @classmethod
  def initializeHandler(cls, serviceInfoDict):
    """ initialize handler """

    try:
      cls.__provenanceDB = ProvenanceDB()
    except RuntimeError as error:
      gLogger.exception(error)
      return S_ERROR(error)

    return S_OK()

  def _parseRes(cls, res):
    if not res['OK']:
      gLogger.error('ProvenanceManager failure', res['Message'])
    return res

  types_addActivityDescription = [six.string_types]

  def export_addActivityDescription(cls, rowJSON):
    '''
    #Insert ActivityDescription
    #:param rowSON
    #:return:
    '''

    rowDict = json.loads(rowJSON)
    res = cls.__provenanceDB.addActivityDescription(rowDict)
    return cls._parseRes(res)

  types_addActivity = [six.string_types]

  def export_addActivity(cls, rowJSON):
    '''
    #Insert ActivityDescription
    #:param rowJSON
    #:return:
    '''

    rowDict = json.loads(rowJSON)
    res = cls.__provenanceDB.addActivity(rowDict)
    return cls._parseRes(res)

  types_addWasAssociatedWith = [six.string_types]

  def export_addWasAssociatedWith(cls, rowJSON):
    '''
    #Insert WasAssociatedWith
    #:param rowJSON
    #:return:
    '''

    rowDict = json.loads(rowJSON)
    res = cls.__provenanceDB.addWasAssociatedWith(rowDict)
    return cls._parseRes(res)

  types_addAgent = [six.string_types]

  def export_addAgent(cls, rowJSON):
    '''
    Insert Agent
    :param rowJSON
    :return:
    '''

    rowDict = json.loads(rowJSON)
    res = cls.__provenanceDB.addAgent(rowDict)
    return cls._parseRes(res)

  types_addDatasetDescription = [six.string_types]

  def export_addDatasetDescription(cls, rowJSON):
    '''
    Insert DatasetDescription
    :param rowJSON
    :return:
    '''

    rowDict = json.loads(rowJSON)
    res = cls.__provenanceDB.addDatasetDescription(rowDict)
    return cls._parseRes(res)

  types_addUsageDescription = [six.string_types]

  def export_addUsageDescription(cls, rowJSON):
    '''
    Insert UsageDescription
    :param rowJSON
    :return:
    '''

    rowDict = json.loads(rowJSON)
    res = cls.__provenanceDB.addUsageDescription(rowDict)
    return cls._parseRes(res)

  types_addGenerationDescription = [six.string_types]

  def export_addGenerationDescription(cls, rowJSON):
    '''
    Insert UsageDescription
    :param rowJSON
    :return:
    '''

    rowDict = json.loads(rowJSON)
    res = cls.__provenanceDB.addGenerationDescription(rowDict)
    return cls._parseRes(res)

  types_addValueDescription = [six.string_types]

  def export_addValueDescription(cls, rowJSON):
    '''
    Insert UsageDescription
    :param rowJSON
    :return:
    '''

    rowDict = json.loads(rowJSON)
    res = cls.__provenanceDB.addValueDescription(rowDict)
    return cls._parseRes(res)

  types_addDatasetEntity = [six.string_types]

  def export_addDatasetEntity(cls, rowJSON):
    '''
    Insert DatasetEntity
    :param rowJSON
    :return:
    '''

    rowDict = json.loads(rowJSON)
    res = cls.__provenanceDB.addDatasetEntity(rowDict)
    return cls._parseRes(res)

  types_addWasAttributedTo = [six.string_types]

  def export_addWasAttributedTo(cls, rowJSON):
    '''
    Insert WasAttributedTo
    :param rowJSON
    :return:
    '''

    rowDict = json.loads(rowJSON)
    res = cls.__provenanceDB.addWasAttributedTo(rowDict)
    return cls._parseRes(res)

  types_addUsed = [six.string_types]

  def export_addUsed(cls, rowJSON):
    '''
    Insert Used
    :param rowJSON
    :return:
    '''

    rowDict = json.loads(rowJSON)
    res = cls.__provenanceDB.addUsed(rowDict)
    return cls._parseRes(res)

  types_addWasGeneratedBy = [six.string_types]

  def export_addWasGeneratedBy(cls, rowJSON):
    '''
    Insert WasGeneratedBy
    :param rowJSON
    :return:
    '''

    rowDict = json.loads(rowJSON)
    res = cls.__provenanceDB.addWasGeneratedBy(rowDict)
    return cls._parseRes(res)

  types_addValueEntity = [six.string_types]

  def export_addValueEntity(cls, rowJSON):
    '''
    Insert ValueEntity
    :param rowJSON
    :return:
    '''

    rowDict = json.loads(rowJSON)
    res = cls.__provenanceDB.addValueEntity(rowDict)
    return cls._parseRes(res)

  types_addWasConfiguredBy = [six.string_types]

  def export_addWasConfiguredBy(cls, rowJSON):
    '''
    Insert WasConfiguredBy
    :param rowJSON
    :return:
    '''

    rowDict = json.loads(rowJSON)
    res = cls.__provenanceDB.addWasConfiguredBy(rowDict)
    return cls._parseRes(res)

  types_addParameter = [six.string_types]

  def export_addParameter(cls, rowJSON):
    '''
    Insert Parameter
    :param rowJSON
    :return:
    '''

    rowDict = json.loads(rowJSON)
    res = cls.__provenanceDB.addParameter(rowDict)
    return cls._parseRes(res)

  types_addConfigFile = [six.string_types]

  def export_addConfigFile(cls, rowJSON):
    '''
    Insert ConfigFile
    :param rowJSON
    :return:
    '''

    rowDict = json.loads(rowJSON)
    res = cls.__provenanceDB.addConfigFile(rowDict)
    return cls._parseRes(res)

  types_addParameterDescription = [six.string_types]

  def export_addParameterDescription(cls, rowJSON):
    '''
    Insert ParameterDescription
    :param rowJSON
    :return:
    '''

    rowDict = json.loads(rowJSON)
    res = cls.__provenanceDB.addParameterDescription(rowDict)
    return cls._parseRes(res)

  types_addConfigFileDescription = [six.string_types]

  def export_addConfigFileDescription(cls, rowJSON):
    '''
    Insert ConfigFileDescription
    :param rowJSON
    :return:
    '''

    rowDict = json.loads(rowJSON)
    res = cls.__provenanceDB.addConfigFileDescription(rowDict)
    return cls._parseRes(res)

  types_getAgents = []

  def export_getAgents(cls):
    '''
    Get Agents
    :return:
    '''

    res = cls.__provenanceDB.getAgents()
    return cls._parseRes(res)

  types_getAgentKey = []

  def export_getAgentKey(cls, agent_id):
    '''
    Get Agent Key
    :return:
    '''

    res = cls.__provenanceDB.getAgentKey(agent_id)
    return cls._parseRes(res)

  types_getDatasetEntity = [six.string_types]

  def export_getDatasetEntity(cls, guid):
    '''
    Get DatasetEntity
    :param datasetEntity guid
    :return:
    '''

    res = cls.__provenanceDB.getDatasetEntity(guid)
    return cls._parseRes(res)

  types_updateDatasetEntity = [six.string_types]

  def export_updateDatasetEntity(cls, guid, invalidatedAtTime):
    '''
    Update DatasetEntity
    :param datasetEntity guid invalidatedAtTime
    :return:
    '''

    res = cls.__provenanceDB.updateDatasetEntity(guid, invalidatedAtTime)
    return cls._parseRes(res)

  types_getUsageDescription = []

  def export_getUsageDescription(cls, activityDescription_id, role):
    '''
    Get UsageDescription
    :param activity_id, role
    :return {'id':usageDescription_id, 'entityDescription_id':entityDescription_id}
    '''

    res = cls.__provenanceDB.getUsageDescription(activityDescription_id, role)
    return cls._parseRes(res)

  types_getGenerationDescription = []

  def export_getGenerationDescription(cls, activityDescription_id, role):
    '''
    Get GenerationDescription
    :param activity_id, role
    :return {'id':GenerationDescription_id, 'entityDescription_id':entityDescription_id}
    '''

    res = cls.__provenanceDB.getGenerationDescription(activityDescription_id, role)
    return cls._parseRes(res)

  types_getParameterDescription = []

  def export_getParameterDescription(cls, activityDescription_id, parameter_name):
    '''
    Get ParameterDescription
    :param activity_id, parameter_name
    :return {'id':ParameterDescription_id}
    '''

    res = cls.__provenanceDB.getParameterDescription(activityDescription_id, parameter_name)
    return cls._parseRes(res)

  types_getConfigFileDescription = []

  def export_getConfigFileDescription(cls, activityDescription_id, configFile_name):
    '''
    Get ConfigFileDescription
    :param activity_id, configFile
    :return {'id':ConfigFileDescription_id}
    '''

    res = cls.__provenanceDB.getConfigFileDescription(activityDescription_id, configFile_name)
    return cls._parseRes(res)

  types_getActivityDescriptionKey = []

  def export_getActivityDescriptionKey(cls, activityDescription_name, activityDescription_version):
    '''
    Get ActivityDescriptionKey
    :param activityDescription_name, activityDescription_version
    :return {'internal_key':activityDescription_key}
    '''

    res = cls.__provenanceDB.getActivityDescriptionKey(activityDescription_name, activityDescription_version)
    return cls._parseRes(res)



#!/usr/bin/env python

"""
"""

__RCSID__ = "$Id$"

import os

from DIRAC.Core.Utilities.DIRACScript import DIRACScript as Script
from DIRAC.TransformationSystem.Client.TransformationClient import TransformationClient

@Script()


def main():

  Script.parseCommandLine(ignoreErrors=True)

  dirNameList = []
  tc = TransformationClient()
  #optionList = ['','with-scts-with-moon_']
  optionList = ['']
  particleList = ['gamma', 'proton']
  compilerList = ['gcc83_noOpt', 'gcc83_matchcpu']
  #compilerList = ['gcc83_noOpt']
  for option in optionList:
    for particle in particleList:
      for compiler in compilerList:
        for cersiz in ['3', '5']:
          transName = 'Prod6_Pipeline_Paranal_%s_North_20_cersiz_%s_%s_%s01' % (particle, cersiz, compiler, option)
          dirName = 'Paranal_%s_North_20_cersiz_%s_%s_%s01' % (particle, cersiz, compiler, option)
          res = tc.getTransformation(transName)
          transID = res['Value']['TransformationID']
          jobGroup='0000' + str(transID)
          dirNameList.append(os.path.join(dirName,jobGroup))
          os.system('dirac-wms-job-get-output -g %s -D %s' % (jobGroup, dirName))


if __name__ == "__main__":
  main()



"""
   Wrapper around the job class to build a workflow element (production step + job)
"""

__RCSID__ = "$Id$"

# generic imports
from copy import deepcopy
import json

# DIRAC imports
import DIRAC
from CTADIRAC.Interfaces.API.EvnDispSingJob import EvnDispSingJob
from CTADIRAC.ProductionSystem.Client.WorkflowElement import WorkflowElement


class EvnDispElement(WorkflowElement):
    """Composite class for workflow element (production step + job)"""

    #############################################################################

    def __init__(self, parent_prod_step):
        """Constructor"""
        WorkflowElement.__init__(self, parent_prod_step)
        self.job = EvnDispSingJob(cpuTime=259200.0)
        self.job.setOutputSandbox(["*Log.txt"])
        self.job.input_limit = None
        self.prod_step.Type = "DataReprocessing"
        self.prod_step.Name = "EvnDisp"
        self.mandatory_keys = {"MCCampaign", "configuration_id", "version"}
        self.constrained_job_keys = {"catalogs", "group_size", "moon"}
        self.constrained_input_keys = {"pointing_dir", "zenith_angle", "sct", "moon"}
        self.file_meta_fields = {"nsb", "split"}

    def set_constrained_job_attribute(self, key, value):
        """Set job attribute with constraints"""
        if key == "catalogs":
            # remove whitespaces between catalogs if there are some and separate between commas
            setattr(self.job, key, json.dumps(value.replace(", ", ",").split(sep=",")))
        elif key == "group_size":
            setattr(self.job, key, value)
            self.prod_step.GroupSize = self.job.group_size
        elif key == "moon":
            if value == "dark":
                self.job.output_file_metadata["nsb"] = 1
            elif value == "half":
                self.job.output_file_metadata["nsb"] = 5
            elif value == "full":
                self.job.output_file_metadata["nsb"] = 19

    def set_constrained_input_query(self, key, value):
        """Set input meta query with constraints"""
        if key == "pointing_dir":
            if value == "North":
                self.prod_step.Inputquery["phiP"] = 180
            elif value == "South":
                self.prod_step.Inputquery["phiP"] = 0
        elif key == "zenith_angle":
            self.prod_step.Inputquery["thetaP"] = float(value)
        elif key == "sct":
            self.prod_step.Inputquery["sct"] = str(value)
        elif key == "moon":
            if value == "dark":
                self.prod_step.Inputquery["nsb"] = 1
            elif value == "half":
                self.prod_step.Inputquery["nsb"] = 5
            elif value == "full":
                self.prod_step.Inputquery["nsb"] = 19

    def build_job_output_data(self, workflow_step):
        """Build job output meta data"""
        metadata = deepcopy(self.prod_step.Inputquery)
        for key, value in workflow_step["job_config"].items():
            metadata[key] = value
        self.job.set_output_metadata(metadata)

    def build_element_config(self):
        """Set job and production step attributes specific to the configuration"""
        self.prod_step.GroupSize = self.job.group_size
        self.job.set_executable_sequence(debug=False)
        self.prod_step.Body = self.job.workflow.toXML()



'''
Launcher script to mark corrupted DL0 files with nsb=-1 on the WMS or create a Transformation.
https://forge.in2p3.fr/issues/48807

Usage:
python prod_mark_corrupted_DL0.py TS <trans_prefix> <dataset name or ascii file with a list of datasets> <group_size>

In testing mode (WMS):
python prod_mark_corrupted_DL0.py WMS

Arguments:
mode: WMS for testing, TS for production
Arguments with TS mode:
trans_prefix: prefix to add to the name of the transformation
input_dataset: name of the input dataset
group_size: n files to process

Example:
python prod_mark_corrupted_DL0.py TS Mark Prod5b_Paranal_AdvancedBaseline_NSB1x_proton_South_40deg_DL0 10
'''

import os

import DIRAC
from DIRAC import gLogger
from DIRAC.Core.Base import Script
Script.parseCommandLine(ignoreErrors=True)

from CTADIRAC.Core.Utilities.tool_box import read_inputs_from_file
from DIRAC.TransformationSystem.Client.Transformation import Transformation
from DIRAC.Interfaces.API.Job import Job
from DIRAC.Core.Workflow.Parameter import Parameter
from DIRAC.Interfaces.API.Dirac import Dirac
from CTADIRAC.Core.Utilities.tool_box import get_dataset_MQ


def submit_trans(job, trans_name, input_meta_query, group_size):
  """ Create a transformation executing the job workflow
  """
  gLogger.notice('submit_trans : %s' % trans_name)

  # Initialize JOB_ID
  job.workflow.addParameter(Parameter("JOB_ID", "000000", "string", "", "",
                                      True, False, "Temporary fix"))

  trans = Transformation()
  trans.setTransformationName(trans_name)  # this must be unique
  trans.setType("DataReprocessing")
  trans.setDescription("Mark corrupted DL0 files")
  trans.setLongDescription("Mark corrupted DL0 files")  # mandatory
  trans.setBody(job.workflow.toXML())
  trans.setGroupSize(group_size)
  trans.setInputMetaQuery(input_meta_query)
  result = trans.addTransformation()  # transformation is created here
  if not result['OK']:
    return result
  trans.setStatus("Active")
  trans.setAgentType("Automatic")
  trans_id = trans.getTransformationID()
  return trans_id

def submit_wms(job):
  """ Submit the job to the WMS
  @todo launch job locally
  """
  dirac = Dirac()
  input_data = ['/vo.cta.in2p3.fr/MC/PROD5b/Paranal/proton/sim_telarray/2455/Data/022xxx/proton_40deg_180deg_run22498___cta-prod5b-paranal_desert-2147m-Paranal-dark.simtel.zst',
           '/vo.cta.in2p3.fr/MC/PROD5b/Paranal/proton/sim_telarray/2455/Data/033xxx/proton_40deg_180deg_run33920___cta-prod5b-paranal_desert-2147m-Paranal-dark.simtel.zst',
           '/vo.cta.in2p3.fr/MC/PROD5b/Paranal/proton/sim_telarray/2455/Data/000xxx/proton_40deg_180deg_run100___cta-prod5b-paranal_desert-2147m-Paranal-dark.simtel.zst']
  job.setInputData(input_data)
  job.setJobGroup('Mark corrupted DL0 files')
  result = dirac.submitJob(job)
  if result['OK']:
    gLogger.notice('Submitted job:', result['Value'])
  return result

def launch_job(args):
  """ Simple launcher to instanciate a Job and setup parameters
      from positional arguments given on the command line.

      Parameters:
      args -- mode (trans_name dataset_name group_size)
  """
  # get arguments
  mode = args[0]

  if mode == 'TS':
    name_prefix = args[1]
    group_size = int(args[3])

    dataset_list = []
    if os.path.isfile(args[2]):
      gLogger.notice('Reading datasets from input file:', args[2])
      dataset_list = read_inputs_from_file(args[2])
    else:
      dataset_list.append(args[2])


  job = Job()
  job.setName('Mark corrupted DL0 files')
  # output
  job.setOutputSandbox(['*Log.txt'])
  package = 'corsika_simtelarray'
  version = '2020-06-29b'
  compiler = 'gcc83_matchcpu'
  job.setType('Reprocessing_BySE')

  # Set executables
  job.setExecutable('cta-prod-setup-software',
                                 arguments='-p %s -v %s -a simulations -g %s' %
                                 (package, version, compiler),
                                 logFile='SetupSoftware_Log.txt')
  job.setExecutable('cta-prod-mark-corrupted-DL0',logFile='MarkCorruptedDL0_Log.txt')
  job.setExecutable('/bin/ls -l', modulesList=['Script', 'FailoverRequest'])

  if mode == 'WMS':
    result = submit_wms(job)
  elif mode == 'TS':
    for dataset_name in dataset_list:
      input_meta_query = get_dataset_MQ(dataset_name)
      trans_name = name_prefix + '_' + dataset_name
      result = submit_trans(job, trans_name, input_meta_query, group_size)
  else:
    gLogger.error('1st argument should be the job mode: WMS or TS,\n\
                             not %s' % mode)
    return None

  return result


#########################################################
if __name__ == '__main__':
  arguments = Script.getPositionalArgs()
  if len(arguments) not in [1, 4]:
    Script.showHelp()
  try:
    result = launch_job(arguments)
    if not result['OK']:
      gLogger.error(result['Message'])
      DIRAC.exit(-1)
    else:
      gLogger.notice('Done')
  except Exception:
    gLogger.exception()
    DIRAC.exit(-1)



"""
  Simple Wrapper on the Job class to handle Simtel Analysis

  https://forge.in2p3.fr/issues/33932
                        June 27th 2018 - J. Bregeon, C. Bigongiari
"""

__RCSID__ = "$Id$"

# generic imports
import os
import json
import collections
# DIRAC imports
from DIRAC.Interfaces.API.Job import Job
from DIRAC.Resources.Catalog.FileCatalogClient import FileCatalogClient


class SimtelTSJob(Job):
  """ Job extension class for Simtel Analysis

  """

  def __init__(self, cpuTime=432000):
    """ Constructor

    Keyword arguments:
    cpuTime -- max cpu time allowed for the job
    """
    Job.__init__(self)
    self.setCPUTime(cpuTime)
    # defaults
    self.setName('Simtel')
    self.package = 'corsika_simhessarray'
    self.program_category = 'tel_sim'
    self.version = '2018-06-12'
    self.configuration_id = 4
    self.output_data_level = 0
    self.base_path = '/vo.cta.in2p3.fr/user/c/ciro.bigongiari/Miniarray15/Simtel'
    self.fcc = FileCatalogClient()
    self.metadata = collections.OrderedDict()
    self.filemetadata = {}
    self.catalogs = json.dumps(['DIRACFileCatalog', 'TSCatalog'])
    self.ts_task_id = 1000
    self.simtel_config_file = 'ASTRI_MiniArray15_Paranal_ACDC_2018_06_12.cfg'
    self.thetaP = 20.0
    self.phiP = 0.0
    self.particle = 'Proton'
    self.output_path = os.path.join(self.base_path, self.particle, str(self.phiP))
    self.se_list = json.dumps(['FRASCATI-USER', 'CNAF-USER'])

  def get_output_path(self):
    """ Recompute the output path where to upload data
    """
    return os.path.join(self.base_path, self.particle, str(self.phiP))

  def setupWorkflow(self, debug=False):
    """ Setup job workflow by defining the sequence of all executables
        All parameters shall have been defined before that method is called.
    """
    # step 1 -- debug
    iStep = 1
    if debug:
      lsStep = self.setExecutable('/bin/ls -alhtr',
                                  logFile='LS_Init_Log.txt')
      lsStep['Value']['name'] = 'Step%i_LS_Init' % iStep
      lsStep['Value']['descr_short'] = 'list files in working directory'
      iStep += 1

    # step 2
    swStep = self.setExecutable('cta-prod3-setupsw',
                                arguments='%s %s' % (self.package,
                                                     self.version),
                                logFile='SetupSoftware_Log.txt')
    swStep['Value']['name'] = 'Step%i_SetupSoftware' % iStep
    swStep['Value']['descr_short'] = 'Setup software'
    iStep += 1

    # step 3 - running
    evStep = self.setExecutable('./dirac_simtel_zstd.sh',
                                arguments="%s TELESCOPE_THETA=%s \
                                    TELESCOPE_PHI=%s" %  # --taskid %s" %
                                (self.simtel_config_file, self.thetaP, self.phiP),
                                logFile='Simtel_Log.txt')
    evStep['Value']['name'] = 'Step%i_Simtel' % iStep
    evStep['Value']['descr_short'] = 'Run Simtel'
    iStep += 1

    # step 4 - running
    # outputpattern = args[0]
    # outputpath = args[1]
    # SEList = json.loads( args[2] )
    # simtel.log.tar simtel.zst.tar
    output_pattern = '*.simtel.zst.tar'
    dmStep = self.setExecutable('../CTADIRAC/Core/scripts/cta-user-managedata.py',
                                arguments="'%s' %s '%s'" %
                                (output_pattern, self.get_output_path(),
                                 self.se_list),
                                logFile='Upload_output_Log.txt')
    dmStep['Value']['name'] = 'DM_Step%i' % iStep
    dmStep['Value']['descr_short'] = 'Upload output data'
    iStep += 1

    # step 5 -- to be removed -- debug only
    if debug:
      lsStep = self.setExecutable('/bin/ls -alhtr',
                                  logFile='LS_End_Log.txt')
      lsStep['Value']['name'] = 'Step%i_LS_End' % iStep
      lsStep['Value']['descr_short'] = 'list files in working directory and in Data directory'
      iStep += 1



"""
   Wrapper around the job class to build a workflow element (production step + job)
"""

__RCSID__ = "$Id$"

# generic imports
import json
from copy import deepcopy

# DIRAC imports
import DIRAC
from CTADIRAC.Interfaces.API.MCPipeNSBJob import MCPipeNSBJob
from CTADIRAC.ProductionSystem.Client.WorkflowElement import WorkflowElement


class SimulationElement(WorkflowElement):
    """Composite class for workflow element (production step + job)"""

    #############################################################################

    def __init__(self, parent_prod_step):
        """Constructor"""
        WorkflowElement.__init__(self, parent_prod_step)
        self.job = MCPipeNSBJob()
        self.job.setOutputSandbox(["*Log.txt"])
        self.prod_step.Type = "MCSimulation"
        self.prod_step.Name = "MC_Simulation"
        self.mandatory_keys = {"MCCampaign", "configuration_id", "version"}
        self.constrained_job_keys = {
            "version",
            "catalogs",
            "sct",
            "particle",
            "pointing_dir",
            "moon",
            "magic",
        }
        self.file_meta_fields = {"nsb", "split"}

    def set_constrained_job_attribute(self, key, value):
        """Set job attribute with constraints"""
        if key == "catalogs":
            # remove whitespaces between catalogs if there are some and separate between commas
            setattr(self.job, key, json.dumps(value.replace(", ", ",").split(sep=",")))
        elif key == "version":
            if "-sc" not in str(
                value
            ):  # if the version has not already been set by 'sct'
                self.job.version = str(value)
        elif key == "particle":
            self.job.set_particle(value)
        elif key == "pointing_dir":
            self.job.set_pointing_dir(value)
        elif key == "sct":
            self.job.set_sct(value)
        elif key == "moon":
            self.job.set_moon(value.replace(", ", ",").split(sep=","))
        elif key == "magic":
            self.job.set_magic(value)

    def build_input_data(self, workflow_config, workflow_step):
        """Simulation Elements do not have input data"""
        self.prod_step.Inputquery = {}

    def build_output_data(self):
        """Build output data from the job metadata and the metadata added on the files"""
        self.prod_step.Outputquery = deepcopy(self.job.output_metadata)
        for key, value in self.job.output_file_metadata.items():
            self.prod_step.Outputquery[key] = value
        if self.job.moon == "--with-full-moon":
            self.prod_step.Outputquery["nsb"] = {"in": [1, 5, 19]}
        elif self.job.moon == "--with-half-moon":
            self.prod_step.Outputquery["nsb"] = {"in": [1, 5]}
        else:
            self.prod_step.Outputquery["nsb"] = {"in": [1]}

    #############################################################################



"""
  New job class to run Prod5b full moon SST only simulations
  Only Paranal in the Alpha layout is supported, using zstd compression
  for simtel output files
          OG, January 2022
"""

__RCSID__ = "$Id$"

# generic imports
import json
import collections
# DIRAC imports
import DIRAC
from CTADIRAC.Interfaces.API.Prod5MCPipeNSBJob import Prod5MCPipeNSBJob


class Prod5bMCPipeAlphaSSTsNSBJob(Prod5MCPipeNSBJob):
  """ Job extension class for Prod5 MC full moon NSB SST only simulations,
  takes care of running corsika piped into simtel
  3 output files are created for Dark, Half Moon and Full Moon NSB
  """

  def __init__(self, cpu_time=259200):
    """ Constructor takes almosst everything from base class

    Keyword arguments:
    cpuTime -- max cpu time allowed for the job
    """
    Prod5MCPipeNSBJob.__init__(self, cpu_time)
    self.setCPUTime(cpu_time)
    self.setName('Prod5bMC_Generation')
    self.version = '2020-06-29b'
    self.compiler = 'gcc83_matchcpu'
    self.configuration_id = 13
    self.n_shower = 100
    self.output_pattern = './Data/*.simtel.zst'
    self.base_path = '/vo.cta.in2p3.fr/MC/PROD5b/'
    self.catalogs = json.dumps(['DIRACFileCatalog', 'TSCatalog'])
    self.n_output_files = 1  # Run only fullmoon NSB (comment lines in multi file)
    self.metadata = collections.OrderedDict()

  def set_site(self, site):
    """ Set the site to simulate

    Parameters:
    site -- a string for the site name (LaPalma)
    """
    if site in ['Paranal']:
      DIRAC.gLogger.info('Set Corsika site to: %s' % site)
      self.cta_site = site
      self.output_pattern = 'Data/*.zst'
    else:
      DIRAC.gLogger.error('Site must be Paranal')
      DIRAC.exit(-1)

  def set_meta_data(self):
    """ define the common meta data of the application
    """
    # The order of the metadata dictionary is important,
    # since it's used to build the directory structure
    self.metadata['array_layout'] = 'Alpha'
    self.metadata['site'] = self.cta_site
    self.metadata['particle'] = self.particle
    # for air shower simulation means North=0 and South=180
    # but here piped into tel_sim so North=180 and South=0
    if self.pointing_dir == 'North':
      self.metadata['phiP'] = 180
    if self.pointing_dir == 'South':
      self.metadata['phiP'] = 0
    self.metadata['thetaP'] = float(self.zenith_angle)
    # hard coded, not ideal... but not sure how to do better
    # self.metadata['airshower_sim_prog'] = 'corsika'
    # self.metadata['airshower_sim_prog_version'] = self.version
    # back to common write up from class
    self.metadata[self.program_category + '_prog'] = self.prog_name
    self.metadata[self.program_category + '_prog_version'] = self.version
    self.metadata['data_level'] = self.output_data_level
    self.metadata['configuration_id'] = self.configuration_id
    # self.metadata['sct'] = False

  def setupWorkflow(self, debug=False):
    """ Override the base class job workflow to adapt to NSB test simulations
        All parameters shall have been defined before that method is called.
    """
    # step 1 - debug only
    i_step = 1
    if debug:
      ls_step = self.setExecutable('/bin/ls -alhtr', logFile='LS_Init_Log.txt')
      ls_step['Value']['name'] = 'Step%i_LS_Init' % i_step
      ls_step['Value']['descr_short'] = 'list files in working directory'
      i_step += 1

      env_step = self.setExecutable('/bin/env', logFile='Env_Log.txt')
      env_step['Value']['name'] = 'Step%i_Env' % i_step
      env_step['Value']['descr_short'] = 'Dump environment'
      i_step += 1

    # step 2
    sw_step = self.setExecutable('cta-prod-setup-software',
                                 arguments='-p %s -v %s -a simulations -g %s' %
                                 (self.package, self.version, self.compiler),
                                 logFile='SetupSoftware_Log.txt')
    sw_step['Value']['name'] = 'Step%i_SetupSoftware' % i_step
    sw_step['Value']['descr_short'] = 'Setup software'
    i_step += 1

    # step 3 runnin
    prod_script = './dirac_prod5_alpha_run'
    if 'gcc10' in self.compiler:
      prod_exe = './dirac_singularity_run'
      prod_args = '%s --start_run %s --run %s %s %s %s %s' % \
                  (prod_script, self.start_run_number, self.run_number,
                   self.cta_site, self.particle, self.pointing_dir,
                   self.zenith_angle)
    else:
      prod_exe = prod_script
      prod_args = '--only-ssts --start_run %s --run %s %s %s %s %s' % \
                  (self.start_run_number, self.run_number,
                   self.cta_site, self.particle, self.pointing_dir,
                   self.zenith_angle)

    cs_step = self.setExecutable(prod_exe, arguments=prod_args,
                                 logFile='CorsikaSimtel_Log.txt')
    cs_step['Value']['name'] = 'Step%i_CorsikaSimtel' % i_step
    cs_step['Value']['descr_short'] = 'Run Corsika piped into simtel'
    i_step += 1

    # step 4a verify Corsika log file
    cl_step = self.setExecutable('cta-prod-verifysteps', arguments='corsika',
                                 logFile='Verify_Corsika_Log.txt')
    cl_step['Value']['name'] = 'Step%i_VerifyCorsikaLog' % i_step
    cl_step['Value']['descr_short'] = 'Verify Corsika log file'
    i_step += 1

    # step 4 verify simtel data
    mgv_step = self.setExecutable('cta-prod-verifysteps',
                                  arguments="generic %0d %0d '%s'" %
                                  (self.n_output_files, self.output_file_size, self.output_pattern),
                                  logFile='Verify_Simtel_Log.txt')
    mgv_step['Value']['name'] = 'Step%i_VerifySimtel' % i_step
    mgv_step['Value']['descr_short'] = 'Verify simtel files'
    i_step += 1

    # step 5 - debug only
    if debug:
      ls_step = self.setExecutable('/bin/ls -Ralhtr', logFile='LS_End_Log.txt')
      ls_step['Value']['name'] = 'Step%i_LS_End' % i_step
      ls_step['Value']['descr_short'] = 'list files in working directory and sub-directory'
      i_step += 1

    # step 6 - define meta data, upload file on SE and register in catalogs
    self.set_meta_data()
    md_json = json.dumps(self.metadata)

    meta_data_field = {'array_layout': 'VARCHAR(128)', 'site': 'VARCHAR(128)',
                       'particle': 'VARCHAR(128)',
                       'phiP': 'float', 'thetaP': 'float',
                       self.program_category + '_prog': 'VARCHAR(128)',
                       self.program_category + '_prog_version': 'VARCHAR(128)',
                       'data_level': 'int', 'configuration_id': 'int', 'merged': 'int'}
    md_field_json = json.dumps(meta_data_field)

    # Upload and register data - NSB=1 dark
    file_meta_data = {'runNumber': self.run_number, 'nsb': 1}
    file_md_json = json.dumps(file_meta_data)
    data_output_pattern = 'Data/*dark*.simtel.zst'

    dm_step = self.setExecutable('cta-prod-managedata',
                                 arguments="'%s' '%s' '%s' %s '%s' %s %s '%s' Data" %
                                 (md_json, md_field_json, file_md_json,
                                  self.base_path, data_output_pattern, self.package,
                                  self.program_category, self.catalogs),
                                 logFile='DataManagement_dark_Log.txt')
    dm_step['Value']['name'] = 'Step%s_DataManagement' % i_step
    dm_step['Value']['descr_short'] = 'Save data files to SE and register them in DFC'
    i_step += 1

    # Upload and register log and histo file - NSB=1
    file_meta_data = {}
    file_md_json = json.dumps(file_meta_data)
    log_file_pattern = 'Data/*dark*.log_hist.tar'
    log_step = self.setExecutable('cta-prod-managedata',
                                  arguments="'%s' '%s' '%s' %s '%s' %s %s '%s' Log" %
                                  (md_json, md_field_json, file_md_json,
                                   self.base_path, log_file_pattern, self.package,
                                   self.program_category, self.catalogs),
                                  logFile='LogManagement_dark_Log.txt')
    log_step['Value']['name'] = 'Step%s_LogManagement' % i_step
    log_step['Value']['descr_short'] = 'Save log to SE and register them in DFC'
    i_step += 1

    # Now switching to half moon NSB
    # Upload and register data - NSB=5 half moon
    file_meta_data = {'runNumber': self.run_number, 'nsb': 5}
    file_md_json = json.dumps(file_meta_data)
    data_output_pattern = 'Data/*-moon*.simtel.zst'

    dm_step = self.setExecutable('cta-prod-managedata',
                                 arguments="'%s' '%s' '%s' %s '%s' %s %s '%s' Data" %
                                 (md_json, md_field_json, file_md_json,
                                  self.base_path, data_output_pattern, self.package,
                                  self.program_category, self.catalogs),
                                 logFile='DataManagement_moon_Log.txt')
    dm_step['Value']['name'] = 'Step%s_DataManagement' % i_step
    dm_step['Value']['descr_short'] = 'Save data files to SE and register them in DFC'
    i_step += 1

    # Upload and register log file - NSB=5
    file_meta_data = {}
    file_md_json = json.dumps(file_meta_data)
    log_file_pattern = 'Data/*-moon*.log_hist.tar'
    log_step = self.setExecutable('cta-prod-managedata',
                                  arguments="'%s' '%s' '%s' %s '%s' %s %s '%s' Log" %
                                  (md_json, md_field_json, file_md_json,
                                   self.base_path, log_file_pattern, self.package,
                                   self.program_category, self.catalogs),
                                  logFile='LogManagement_moon_Log.txt')
    log_step['Value']['name'] = 'Step%s_LogManagement' % i_step
    log_step['Value']['descr_short'] = 'Save log to SE and register them in DFC'
    i_step += 1

    # Now switching to full moon NSB
    # Upload and register data - NSB=19 full moon
    file_meta_data = {'runNumber': self.run_number, 'nsb': 19}
    file_md_json = json.dumps(file_meta_data)
    data_output_pattern = 'Data/*-fullmoon*.simtel.zst'

    dm_step = self.setExecutable('cta-prod-managedata',
                                 arguments="'%s' '%s' '%s' %s '%s' %s %s '%s' Data" %
                                 (md_json, md_field_json, file_md_json,
                                  self.base_path, data_output_pattern, self.package,
                                  self.program_category, self.catalogs),
                                 logFile='DataManagement_fullmoon_Log.txt')
    dm_step['Value']['name'] = 'Step%s_DataManagement' % i_step
    dm_step['Value']['descr_short'] = 'Save data files to SE and register them in DFC'
    i_step += 1

    # Upload and register log file - NSB=19
    file_meta_data = {}
    file_md_json = json.dumps(file_meta_data)
    log_file_pattern = 'Data/*-fullmoon*.log_hist.tar'
    log_step = self.setExecutable('cta-prod-managedata',
                                  arguments="'%s' '%s' '%s' %s '%s' %s %s '%s' Log" %
                                  (md_json, md_field_json, file_md_json,
                                   self.base_path, log_file_pattern, self.package,
                                   self.program_category, self.catalogs),
                                  logFile='LogManagement_fullmoon_Log.txt')
    log_step['Value']['name'] = 'Step%s_LogManagement' % i_step
    log_step['Value']['descr_short'] = 'Save log to SE and register them in DFC'
    i_step += 1

    # Step 6 - debug only
    if debug:
      ls_step = self.setExecutable('/bin/ls -Ralhtr', logFile='LS_End_Log.txt')
      ls_step['Value']['name'] = 'Step%s_LSHOME_End' % i_step
      ls_step['Value']['descr_short'] = 'list files in Home directory'
      i_step += 1

    # Number of showers is passed via an environment variable
    self.setExecutionEnv({'NSHOW': '%s' % self.n_shower})



"""
   Wrapper around the job class to build a workflow element (production step + job)
"""

__RCSID__ = "$Id$"

# generic imports
import json
from copy import deepcopy


# DIRAC imports
import DIRAC
from DIRAC.Interfaces.API.Job import Job
from DIRAC.ProductionSystem.Client.ProductionStep import ProductionStep
from CTADIRAC.Core.Utilities.tool_box import get_dataset_MQ
from DIRAC.Resources.Catalog.FileCatalogClient import FileCatalogClient


class WorkflowElement:
    """Composite class for workflow element (production step + job)"""

    #############################################################################

    def __init__(self, parent_prod_step):
        """Constructor"""
        self.job = Job()
        self.prod_step = ProductionStep()
        self.prod_step.ParentStep = parent_prod_step
        self.fc = FileCatalogClient()
        self.mandatory_keys = {"MCCampaign", "configuration_id"}
        self.constrained_job_keys = {""}
        self.constrained_input_keys = {""}
        self.file_meta_fields = {""}

    def set_constrained_job_attribute(self, key, value):
        """Set job attribute with constraints"""
        setattr(self.job, key, value)

    def set_constrained_input_query(self, key, value):
        """Set input meta query with constraints"""
        self.prod_step.Inputquery[key] = value

    def build_job_attributes_from_job_config(self, workflow_step):
        """Build job attributes with job_config values"""
        for key, value in workflow_step["job_config"].items():
            if value is not None:
                if key in self.constrained_job_keys:
                    self.set_constrained_job_attribute(key, value)
                elif key in self.file_meta_fields:
                    self.job.output_file_metadata[key] = value
                else:
                    setattr(self.job, key, value)
            elif key in self.mandatory_keys:
                DIRAC.gLogger.error("%s is mandatory" % key)
                DIRAC.exit(-1)

    def build_job_attributes_from_common(self, workflow_config):
        """Build job attributes concerning the whole production"""
        for key, value in workflow_config["Common"].items():
            if value is not None:
                setattr(self.job, key, value)
            elif key in self.mandatory_keys:
                DIRAC.gLogger.error("%s is mandatory" % key)
                DIRAC.exit(-1)

    def build_job_attributes_from_input(self):
        """Set job attributes from input meta query"""
        for key, value in self.prod_step.Inputquery.items():
            if key in self.file_meta_fields:
                try:
                    self.job.output_file_metadata[key] = value["="]
                except TypeError:
                    self.job.output_file_metadata[key] = value
            else:
                try:
                    setattr(self.job, key, value["="])
                except TypeError:
                    setattr(self.job, key, value)

    def build_job_attributes(self, workflow_config, workflow_step):
        """Set job attributes"""
        self.build_job_attributes_from_input()
        self.build_job_attributes_from_common(workflow_config)
        self.build_job_attributes_from_job_config(workflow_step)

    def build_input_data(self, workflow_config, workflow_step):
        """Build input data"""
        self.prod_step.Inputquery = {}
        if self.prod_step.ParentStep:
            self.prod_step.Inputquery = self.prod_step.ParentStep.Outputquery
        else:
            if workflow_step["input_meta_query"].get("dataset"):
                self.prod_step.Inputquery = get_dataset_MQ(
                    workflow_step["input_meta_query"]["dataset"]
                )

        # refine parent MQ by adding extra specification if needed
        for key, value in workflow_step["input_meta_query"].items():
            if value is not None:
                if key in self.constrained_input_keys:
                    self.set_constrained_input_query(key, value)
                elif (key != "dataset") and (key != "parentID"):
                    self.prod_step.Inputquery[key] = value

    def build_job_input_data(self, mode):
        if mode.lower() in ["wms", "local"]:
            res = self.fc.findFilesByMetadata(dict(self.prod_step.Inputquery))
            if not res["OK"]:
                DIRAC.gLogger.error(res["Message"])
                DIRAC.exit(-1)
            # Limit the nb of input data to process
            # To do: decide if allow this feature also in production mode
            input_data_limit = self.job.input_limit
            input_data = res["Value"][:input_data_limit]
            self.job.setInputData(input_data)
            if not input_data and mode.lower() == "wms":
                DIRAC.gLogger.error("No job submitted: job must have input data")
                DIRAC.exit(-1)

    def build_element_config(self):
        """Set job and production step attributes specific to the configuration"""
        self.job.set_executable_sequence(debug=False)
        self.prod_step.Body = self.job.workflow.toXML()

    def build_job_output_data(self, workflow_step):
        """Build job output meta data"""
        self.job.set_output_metadata()

    def build_output_data(self):
        """Build output data from the job metadata and the metadata added on the files"""
        self.prod_step.Outputquery = deepcopy(self.job.output_metadata)
        for key, value in self.job.output_file_metadata.items():
            self.prod_step.Outputquery[key] = value


#############################################################################



#!/usr/bin/env python

__RCSID__ = "$Id$"

# generic imports
from multiprocessing import Pool

# DIRAC imports
from DIRAC import gLogger
from CTADIRAC.Core.Utilities.tool_box import read_inputs_from_file
from DIRAC.Resources.Catalog.TSCatalogClient import TSCatalogClient
from DIRAC.Core.Utilities.DIRACScript import DIRACScript as Script


Script.parseCommandLine(ignoreErrors=True)

args = Script.getPositionalArgs()
if len(args) == 1:
  infile = args[0]
else:
  Script.showHelp()

@Script()
def main():
  infileList = read_inputs_from_file(infile)
  p = Pool(10)
  p.map(addfile, infileList)

def addfile(lfn):
  tsCatalogClient = TSCatalogClient()
  res = tsCatalogClient.addFile(lfn)
  if not res['OK']:
    gLogger.error('Error uploading file', lfn)
    return res['Message']

if __name__ == '__main__':
  main()


#!/usr/bin/env python

"""
  Clean an existing transformation

  Usage:
    cta-transformation-clean <transID>
"""

__RCSID__ = "$Id$"

import DIRAC
from DIRAC.Core.Utilities.DIRACScript import DIRACScript as Script
from DIRAC.TransformationSystem.Client.TransformationClient import TransformationClient

@Script()
def main():
    Script.parseCommandLine()
    args = Script.getPositionalArgs()
    if (len(args) != 1):
      Script.showHelp()

    # get arguments
    transID = args[0]

    tc = TransformationClient()
    res = tc.cleanTransformation(transID)

    if not res['OK']:
      DIRAC.gLogger.error(res['Message'])
      DIRAC.exit(-1)
    else:
      DIRAC.gLogger.notice("Successfully cleaned transformation %s" % transID)
      DIRAC.exit(0)

if __name__ == "__main__":
  main()


#!/usr/bin/env python
'''
Move a dataset distributed on on number of production SEs:
'DESY-ZN-Disk', 'LPNHE-Disk', 'CNAF-Disk', 'CYF-STORM-Disk','LAPP-Disk', 'CEA-Disk', 'CC-IN2P3-Disk', 'LANCASTER-Disk', 'POLGRID-Disk'
to a given destination SE, e.g. CC-IN2P3-Tape

Usage:
cta-prod-move-dataset <dataset name or ascii file with a list of datasets> <dest SE>

Optional arguments:
<group size>: size of the transformation (default=1)
<extra_tag>: extra tag to be appended to the name of the transformation

Example:
Prod4_Paranal_gamma_North_20deg_SSTOnly_MC0 CC-IN2P3-Tape 100

Warning:
If the <extra_tag> option is used the transformations will not be found by the cta-prod-move-status command and should be monitored using
the transformation CLI or the Transformation Web Monitor
'''

__RCSID__ = "$Id$"

import os

import DIRAC
from DIRAC import gLogger
from DIRAC.Core.Utilities.DIRACScript import DIRACScript as Script
from DIRAC.Resources.Catalog.FileCatalogClient import FileCatalogClient
from DIRAC.TransformationSystem.Utilities.ReplicationTransformation import createDataTransformation
from CTADIRAC.Core.Utilities.tool_box import read_inputs_from_file

def get_dataset_info(dataset_name):
  """ Return essential dataset information
      Name, number of files, total size and meta query
  """
  fc = FileCatalogClient()
  res = fc.getDatasets(dataset_name)
  if not res['OK']:
    gLogger.error(res['Message'])
    DIRAC.exit(-1)
  dataset_dict = res['Value']
  res = dataset_dict['Successful'][dataset_name][dataset_name]
  number_of_files = res['NumberOfFiles']
  meta_query = res['MetaQuery']
  total_size = res['TotalSize']
  return (dataset_name, number_of_files, total_size, meta_query)

@Script()
def main():
  dataset_list = []
  Script.parseCommandLine(ignoreErrors=True)
  argss = Script.getPositionalArgs()
  if len(argss) < 2:
    Script.showHelp()
  if os.path.isfile(argss[0]):
    gLogger.notice('Reading datasets from input file:', argss[0])
    dataset_list = read_inputs_from_file(argss[0])
  else:
    dataset_list.append(argss[0])

  dest_se = argss[1]
  group_size = 1
  extra_tag = ""

  if len(argss) >= 3:
    group_size = argss[2]
  if len(argss) == 4:
    extra_tag = argss[3]

  # Check input data set information
  for dataset_name in dataset_list:
    name, n_files, size, meta_query = get_dataset_info(dataset_name)
    gLogger.notice('Found dataset %s with %d files.' % (name, n_files))
    gLogger.debug(meta_query)
    # choose a metaKey
    # To do: avoid hard coded metakey
    if 'MCCampaign' in meta_query:
      meta_key = 'MCCampaign'
      meta_value = meta_query['MCCampaign']
    else:
       gLogger.error('Error: MCCampaign metadata not defined')
       DIRAC.exit(-1)

    tag = dataset_name
    if extra_tag:
      tag = tag + '_' + extra_tag

    do_it = True
    # To do: replace hard coded SE with SE read from CS
    source_se = ['DESY-ZN-Disk', 'LPNHE-Disk', 'CNAF-Disk', 'CYF-STORM-Disk',
               'LAPP-Disk', 'CEA-Disk', 'CC-IN2P3-Disk', 'POLGRID-Disk', 'LANCASTER-Disk']

    # create Transformation
    res = createDataTransformation(flavour='Moving',
                                   targetSE=dest_se,
                                   sourceSE=source_se,
                                   metaKey=meta_key,
                                   metaValue=meta_value,
                                   extraData=meta_query,
                                   extraname=tag,
                                   groupSize=int(group_size),
                                   plugin='Broadcast',
                                   tGroup=None,
                                   tBody=None,
                                   enable=do_it,
                                   )
    if not res['OK']:
      gLogger.error(res["Message"])
      DIRAC.exit(-1)

  DIRAC.exit()

#########################################################
if __name__ == '__main__':
  main()



"""
  New job class to run Prod6 for Paranal and La Palma
"""

__RCSID__ = "$Id$"

# generic imports
import json
import collections

# DIRAC imports
import DIRAC
from DIRAC.Interfaces.API.Job import Job
from CTADIRAC.Core.Utilities.tool_box import DATA_LEVEL_METADATA_ID


class MCPipeNSBJob(Job):
    """Job extension class for Prod5 MC NSB simulations,
    takes care of running corsika piped into simtel
    2 output files are created for Dark and Moon NSB
    """

    def __init__(self, cpu_time=259200):
        """Constructor takes almosst everything from base class

        Keyword arguments:
        cpuTime -- max cpu time allowed for the job
        """
        Job.__init__(self)
        self.setCPUTime(cpu_time)
        self.setName("MC_Generation")
        self.setType("MCSimulation")
        self.package = "corsika_simtelarray"
        self.version = "2022-08-03"
        self.compiler = "gcc83_matchcpu"
        self.program_category = "tel_sim"
        self.prog_name = "sim_telarray"
        self.start_run_number = "0"
        self.run_number = "@{JOB_ID}"  # dynamic
        self.configuration_id = 15
        self.output_data_level = DATA_LEVEL_METADATA_ID["R1"]
        self.base_path = "/vo.cta.in2p3.fr/MC/PROD6/"
        self.catalogs = json.dumps(["DIRACFileCatalog", "TSCatalog"])
        self.output_metadata = collections.OrderedDict()
        self.output_file_metadata = collections.OrderedDict()
        self.sct = ""
        self.magic = ""
        self.moon = ""
        self.array_layout = "Prod6-Hyperarray"

    def set_site(self, site):
        """Set the site to simulate

        Parameters:
        site -- a string for the site name (LaPalma)
        """
        if site in ["Paranal", "LaPalma"]:
            DIRAC.gLogger.info("Set Corsika site to: %s" % site)
            self.site = site
        else:
            DIRAC.gLogger.error("Site is unknown: %s" % site)
            DIRAC.exit(-1)

    def set_particle(self, particle):
        """Set the corsika primary particle

        Parameters:
        particle -- a string for the particle type/name
        """
        if particle in ["gamma", "gamma-diffuse", "electron", "proton", "helium"]:
            DIRAC.gLogger.info("Set Corsika particle to: %s" % particle)
            self.particle = particle
        else:
            DIRAC.gLogger.error("Corsika does not know particle type: %s" % particle)
            DIRAC.exit(-1)

    def set_pointing_dir(self, pointing):
        """Set the pointing direction, North or South

        Parameters:
        pointing -- a string for the pointing direction
        """
        if pointing in ["North", "South", "East", "West"]:
            DIRAC.gLogger.info("Set Pointing dir to: %s" % pointing)
            self.pointing_dir = pointing
        else:
            DIRAC.gLogger.error("Unknown pointing direction: %s" % pointing)
            DIRAC.exit(-1)

    def set_moon(self, moon=["dark", "half", "full"]):
        """Set to simulate with various moon conditions

        Parameters:
        moon -- a list of moon conditions for simulation
        """
        if moon == ["dark"]:
            DIRAC.gLogger.info("Set simulations with dark conditions")
            self.moon = ""
        elif moon == ["dark", "half"]:
            DIRAC.gLogger.info("Set simulations with half-moon conditions")
            self.moon = "--with-half-moon"
        elif moon == ["dark", "half", "full"]:
            DIRAC.gLogger.info("Set simulations with full-moon conditions")
            self.moon = "--with-full-moon"
        else:
            DIRAC.gLogger.error(
                "Unknown moon option: %s. Options for simulation step are: \n dark \n dark, half \n dark, half, full"
                % moon
            )
            DIRAC.exit(-1)

    def set_magic(self, with_magic=False):
        """Set to simulate with MAGIC

        Parameters:
        with_magic -- a boolean for simulating with MAGIC
        """
        if with_magic is True:
            DIRAC.gLogger.info("Set simulations with MAGIC telescopes")
            self.magic = "--with-magic"

    def set_sct(self, with_sct=None):
        """Set to include SCTs in simulations

        Parameters:
        with_sct -- a string to include SCTs
        """
        if with_sct is not None:
            if with_sct.lower() == "all":
                DIRAC.gLogger.info("Set to include SCTs for all MST positions")
                self.sct = "--with-all-scts"
            elif with_sct.lower() == "non-alpha":
                DIRAC.gLogger.info("Set to include SCTs for non-Alpha MST positions")
                self.sct = "--with-sct"
            self.version = self.version + "-sc"

    def set_output_metadata(self):
        """define the common meta data of the application"""
        # The order of the metadata dictionary is important,
        # since it's used to build the directory structure
        self.output_metadata["array_layout"] = self.array_layout
        self.output_metadata["site"] = self.site
        self.output_metadata["particle"] = self.particle
        # for air shower simulation means North=0 and South=180
        # but here piped into tel_sim so North=180 and South=0
        if self.pointing_dir == "North":
            self.output_metadata["phiP"] = 180
        if self.pointing_dir == "South":
            self.output_metadata["phiP"] = 0
        self.output_metadata["thetaP"] = float(self.zenith_angle)
        if self.sct:
            self.output_metadata["sct"] = "True"
        else:
            self.output_metadata["sct"] = "False"
        self.output_metadata[self.program_category + "_prog"] = self.prog_name
        self.output_metadata[self.program_category + "_prog_version"] = "".join(
            self.version.rsplit("-sc", 1)
        )
        self.output_metadata["data_level"] = self.output_data_level
        self.output_metadata["configuration_id"] = self.configuration_id
        self.output_metadata["merged"] = 0
        self.output_metadata["MCCampaign"] = self.MCCampaign

    def set_executable_sequence(self, debug=False):
        """Override the base class job workflow to adapt to NSB test simulations
        All parameters shall have been defined before that method is called.
        """
        # step 1 - debug only
        i_step = 1
        if debug:
            ls_step = self.setExecutable("/bin/ls -alhtr", logFile="LS_Init_Log.txt")
            ls_step["Value"]["name"] = "Step%i_LS_Init" % i_step
            ls_step["Value"]["descr_short"] = "list files in working directory"
            i_step += 1

            env_step = self.setExecutable("/bin/env", logFile="Env_Log.txt")
            env_step["Value"]["name"] = "Step%i_Env" % i_step
            env_step["Value"]["descr_short"] = "Dump environment"
            i_step += 1

        # step 2 : use new CVMFS repo
        sw_step = self.setExecutable(
            "cta-prod-setup-software",
            arguments="-p %s -v %s -a simulations -g %s"
            % (self.package, self.version, self.compiler),
            logFile="SetupSoftware_Log.txt",
        )
        sw_step["Value"]["name"] = "Step%i_SetupSoftware" % i_step
        sw_step["Value"]["descr_short"] = "Setup software"
        i_step += 1

        # step 3 run corsika+sim_telarray
        prod_exe = "./dirac_prod6_run"
        prod_args = "--start_run %s --run %s %s %s %s %s %s %s %s" % (
            self.start_run_number,
            self.run_number,
            self.moon,
            self.sct,
            self.magic,
            self.site,
            self.particle,
            self.pointing_dir,
            self.zenith_angle,
        )

        cs_step = self.setExecutable(
            prod_exe, arguments=prod_args, logFile="CorsikaSimtel_Log.txt"
        )
        cs_step["Value"]["name"] = "Step%i_CorsikaSimtel" % i_step
        cs_step["Value"]["descr_short"] = "Run Corsika piped into simtel"
        i_step += 1

        # step 4 verify the number of events in the simtel file
        data_output_pattern = "Data/*.simtel.zst"
        mgv_step = self.setExecutable(
            "dirac_simtel_check",
            arguments="'%s'" % (data_output_pattern),
            logFile="Verify_n_showers_Log.txt",
        )
        mgv_step["Value"]["name"] = "Step%i_VerifyNShowers" % i_step
        mgv_step["Value"]["descr_short"] = "Verify number of showers"
        i_step += 1

        # step 5 - define meta data, upload file on SE and register in catalogs

        md_json = json.dumps(self.output_metadata)

        meta_data_field = {
            "array_layout": "VARCHAR(128)",
            "site": "VARCHAR(128)",
            "particle": "VARCHAR(128)",
            "phiP": "float",
            "thetaP": "float",
            "sct": "VARCHAR(128)",
            self.program_category + "_prog": "VARCHAR(128)",
            self.program_category + "_prog_version": "VARCHAR(128)",
            "data_level": "int",
            "configuration_id": "int",
            "merged": "int",
        }
        md_field_json = json.dumps(meta_data_field)

        # Upload and register data - NSB=1 dark
        file_meta_data = {
            "runNumber": self.run_number,
            "nsb": 1,
        }
        file_md_json = json.dumps(file_meta_data)
        data_output_pattern = "Data/*dark*.simtel.zst"

        dm_step = self.setExecutable(
            "cta-prod-managedata",
            arguments="'%s' '%s' '%s' %s '%s' %s %s '%s' Data"
            % (
                md_json,
                md_field_json,
                file_md_json,
                self.base_path,
                data_output_pattern,
                self.package,
                self.program_category,
                self.catalogs,
            ),
            logFile="DataManagement_dark_Log.txt",
        )
        dm_step["Value"]["name"] = "Step%s_DataManagement" % i_step
        dm_step["Value"][
            "descr_short"
        ] = "Save data files to SE and register them in DFC"
        i_step += 1

        # Upload and register log and histo file - NSB=1
        file_meta_data = {}
        file_md_json = json.dumps(file_meta_data)
        log_file_pattern = "Data/*dark*.log_hist.tar"
        log_step = self.setExecutable(
            "cta-prod-managedata",
            arguments="'%s' '%s' '%s' %s '%s' %s %s '%s' Log"
            % (
                md_json,
                md_field_json,
                file_md_json,
                self.base_path,
                log_file_pattern,
                self.package,
                self.program_category,
                self.catalogs,
            ),
            logFile="LogManagement_dark_Log.txt",
        )
        log_step["Value"]["name"] = "Step%s_LogManagement" % i_step
        log_step["Value"]["descr_short"] = "Save log to SE and register them in DFC"
        i_step += 1

        if self.moon:  # At least half moon
            # Now switching to half moon NSB
            # Upload and register data - NSB=5 half moon
            file_meta_data = {
                "runNumber": self.run_number,
                "nsb": 5,
            }
            file_md_json = json.dumps(file_meta_data)
            data_output_pattern = "Data/*-moon*.simtel.zst"

            dm_step = self.setExecutable(
                "cta-prod-managedata",
                arguments="'%s' '%s' '%s' %s '%s' %s %s '%s' Data"
                % (
                    md_json,
                    md_field_json,
                    file_md_json,
                    self.base_path,
                    data_output_pattern,
                    self.package,
                    self.program_category,
                    self.catalogs,
                ),
                logFile="DataManagement_moon_Log.txt",
            )
            dm_step["Value"]["name"] = "Step%s_DataManagement" % i_step
            dm_step["Value"][
                "descr_short"
            ] = "Save data files to SE and register them in DFC"
            i_step += 1

            # Upload and register log file - NSB=5
            file_meta_data = {}
            file_md_json = json.dumps(file_meta_data)
            log_file_pattern = "Data/*-moon*.log_hist.tar"
            log_step = self.setExecutable(
                "cta-prod-managedata",
                arguments="'%s' '%s' '%s' %s '%s' %s %s '%s' Log"
                % (
                    md_json,
                    md_field_json,
                    file_md_json,
                    self.base_path,
                    log_file_pattern,
                    self.package,
                    self.program_category,
                    self.catalogs,
                ),
                logFile="LogManagement_moon_Log.txt",
            )
            log_step["Value"]["name"] = "Step%s_LogManagement" % i_step
            log_step["Value"]["descr_short"] = "Save log to SE and register them in DFC"
            i_step += 1

            if self.moon == "--with-full-moon":
                # Now switching to full moon NSB
                # Upload and register data - NSB=19 full moon
                file_meta_data = {
                    "runNumber": self.run_number,
                    "nsb": 19,
                }
                file_md_json = json.dumps(file_meta_data)
                data_output_pattern = "Data/*-fullmoon*.simtel.zst"

                dm_step = self.setExecutable(
                    "cta-prod-managedata",
                    arguments="'%s' '%s' '%s' %s '%s' %s %s '%s' Data"
                    % (
                        md_json,
                        md_field_json,
                        file_md_json,
                        self.base_path,
                        data_output_pattern,
                        self.package,
                        self.program_category,
                        self.catalogs,
                    ),
                    logFile="DataManagement_fullmoon_Log.txt",
                )
                dm_step["Value"]["name"] = "Step%s_DataManagement" % i_step
                dm_step["Value"][
                    "descr_short"
                ] = "Save data files to SE and register them in DFC"
                i_step += 1

                # Upload and register log file - NSB=19
                file_meta_data = {}
                file_md_json = json.dumps(file_meta_data)
                log_file_pattern = "Data/*-fullmoon*.log_hist.tar"
                log_step = self.setExecutable(
                    "cta-prod-managedata",
                    arguments="'%s' '%s' '%s' %s '%s' %s %s '%s' Log"
                    % (
                        md_json,
                        md_field_json,
                        file_md_json,
                        self.base_path,
                        log_file_pattern,
                        self.package,
                        self.program_category,
                        self.catalogs,
                    ),
                    logFile="LogManagement_fullmoon_Log.txt",
                )
                log_step["Value"]["name"] = "Step%s_LogManagement" % i_step
                log_step["Value"][
                    "descr_short"
                ] = "Save log to SE and register them in DFC"
                i_step += 1

        # Step 6 - debug only
        if debug:
            ls_step = self.setExecutable("/bin/ls -Ralhtr", logFile="LS_End_Log.txt")
            ls_step["Value"]["name"] = "Step%s_LSHOME_End" % i_step
            ls_step["Value"]["descr_short"] = "list files in Home directory"
            i_step += 1

        # Number of showers is passed via an environment variable
        self.setExecutionEnv({"NSHOW": "%s" % self.n_shower})



"""
  Wrapper on the Job class to handle a simple ctapipe job using provenance
"""

__RCSID__ = "$Id$"

# generic imports
import json
import collections

# DIRAC imports
from DIRAC.Interfaces.API.Job import Job
from CTADIRAC.Core.Utilities.tool_box import DATA_LEVEL_METADATA_ID


class ProvCtapipeJob(Job):
  """ Job extension class for provenance test with ctapipe
  """

  def __init__(self, cpuTime=36000):
    """ Constructor

    Keyword arguments:
    cpuTime -- max cpu time allowed for the job
    """
    Job.__init__(self)
    self.setCPUTime(cpuTime)
    # defaults
    self.setName('prov_test')
    self.package = 'provtest'
    self.program_category = 'calibimgreco'
    self.version = 'v0.5.3'
    self.configuration_id = 1  # To be set according to the input dataset
    ########################################################################
    # data management params
    self.output_type = 'DL1'
    self.basepath = '/vo.cta.in2p3.fr/user/a/arrabito'
    self.metadata = collections.OrderedDict()
    self.filemetadata = {}
    self.catalogs = json.dumps(['DIRACFileCatalog', 'TSCatalog'])

  def setCtapipeMD(self, simtelMD):
    """ Set ctapipe meta data

    Parameters:
    simtelMD -- metadata dictionary as for simtel
    """
    # set ctapipe directory meta data
    self.metadata['array_layout'] = simtelMD['array_layout']
    self.metadata['site'] = simtelMD['site']
    self.metadata['particle'] = simtelMD['particle']
    try:
      phiP = simtelMD['phiP']['=']
    except BaseException:
      phiP = simtelMD['phiP']
    self.metadata['phiP'] = phiP
    try:
      thetaP = simtelMD['thetaP']['=']
    except BaseException:
      thetaP = simtelMD['thetaP']
    self.metadata['thetaP'] = thetaP
    self.metadata[self.program_category + '_prog'] = self.package
    self.metadata[self.program_category + '_prog_version'] = self.version
    self.metadata['data_level'] = DATA_LEVEL_METADATA_ID[self.output_type]
    self.metadata['configuration_id'] = self.configuration_id

  def setupWorkflow(self, debug=False):
    """ Setup job workflow by defining the sequence of all executables
        All parameters shall have been defined before that method is called
    """

    # step 1 -- to be removed -- debug only
    iStep = 1
    if debug:
      step = self.setExecutable('/bin/ls -alhtr',
                                logFile='LS_Init_Log.txt')
      step['Value']['name'] = 'Step%i_LS_Init' % iStep
      step['Value']['descr_short'] = 'list files in working directory'
      iStep += 1

    # step 2
    step = self.setExecutable('cta-prod3-setupsw',
                              arguments='%s %s %s' %
                              (self.package, self.version, self.program_category),
                              logFile='SetupSoftware_Log.txt')
    step['Value']['name'] = 'Step%i_SetupSoftware' % iStep
    step['Value']['descr_short'] = 'Setup software'
    iStep += 1

    # step 2bis
    # arguments are nbFiles=0 (not used) and fileSize=100kB
    step = self.setExecutable('cta-prod-verifysteps',
                              arguments='analysisinputs 0 100',
                              logFile='Verify_CtapipeInputs_Log.txt')
    step['Value']['name'] = 'Step%i_VerifyPipeInputs' % iStep
    step['Value']['descr_short'] = 'Verify ctapipe Inputs'
    iStep += 1

    # step 3: run MuonRecProv_step1 (run the reco algo)
    ctapipe_exe = './dirac_provctapipe'
    self.setExecutable(ctapipe_exe, logFile='Ctapipe_Log.txt')
    step['Value']['name'] = 'Step%i_Ctapipe' % iStep
    step['Value']['descr_short'] = 'Run Ctapipe'
    iStep += 1

    # step 4
    # ## the order of the metadata dictionary is important, since it's used to build the directory structure
    mdjson = json.dumps(self.metadata)
    metadatafield = {'array_layout': 'VARCHAR(128)',
                     'site': 'VARCHAR(128)',
                     'particle': 'VARCHAR(128)', 'phiP': 'float',
                     'thetaP': 'float',
                     self.program_category + '_prog': 'VARCHAR(128)',
                     self.program_category + '_prog_version': 'VARCHAR(128)',
                     'data_level': 'int', 'configuration_id': 'int', 'merged': 'int'}
    mdfieldjson = json.dumps(metadatafield)

    # register Data
    outputpattern = './*.hdf5'
    file_md_json = json.dumps(self.filemetadata)

    scripts = '../CTADIRAC/Core/scripts'
    step = self.setExecutable(scripts + '/cta-prod-managedata.py',
                              arguments="'%s' '%s' '%s' %s '%s' %s %s '%s' %s" %
                              (mdjson, mdfieldjson, file_md_json, self.basepath,
                               outputpattern, self.package, self.program_category,
                               self.catalogs, "Data"),
                              logFile='DataManagement_Log.txt')
    step['Value']['name'] = 'Step%i_DataManagement' % iStep
    step['Value']['descr_short'] = 'Save data files to SE and register them in DFC'
    iStep += 1

    # step 5: add Prov Data
    self.setExecutable('cta-prod-add-prov', logFile='AddProvData_Log.txt')
    step['Value']['name'] = 'Step%i_Prov' % iStep
    step['Value']['descr_short'] = 'Run Prov'
    iStep += 1

    # step 6 -- to be removed -- debug only
    if debug:
      step = self.setExecutable('/bin/ls -alhtr ./ ',
                                logFile='LS_End_Log.txt')
      step['Value']['name'] = 'Step%i_LS_End' % iStep
      step['Value']['descr_short'] = 'list files in working and Data directory'
      iStep += 1



#!/usr/bin/env python
"""
Show the progress of DataProcessing transformations

Usage:
   cta-transformation-show-progress <transID or ascii file with a list of transID comma separated>

Example:
   cta-transformation-show-progress 2010
"""

__RCSID__ = "$Id$"

import os

import DIRAC
from DIRAC import gLogger
from DIRAC.Core.Utilities.DIRACScript import DIRACScript as Script
from DIRAC.TransformationSystem.Client.TransformationClient import TransformationClient


def get_transformation_info(transID):

  transClient = TransformationClient()

  paramShowNames = [
      "TransformationID",
      "TransformationName",
      "Type",
      "Status",
      "Files_Total",
      "Files_PercentProcessed",
      "Files_Processed",
      "Files_Unused",
      "Jobs_TotalCreated",
      "Jobs_Waiting",
      "Jobs_Running",
      "Jobs_Done",
      "Jobs_Failed",
      "Jobs_Stalled",
  ]

  res = transClient.getTransformationSummaryWeb({"TransformationID": transID}, [], 0, 1)

  if not res["OK"]:
      DIRAC.gLogger.error(res["Message"])
      DIRAC.exit(-1)

  if res["Value"]["TotalRecords"] > 0:
    paramNames = res["Value"]["ParameterNames"]
    for paramValues in res["Value"]["Records"]:
      paramShowValues = map(lambda pname: paramValues[paramNames.index(pname)], paramShowNames)
      showDict = dict(zip(paramShowNames, paramShowValues))
      files_PercentProcessed = showDict["Files_PercentProcessed"]
      transName = showDict["TransformationName"]

  return (transName, files_PercentProcessed)

#########################################################
@Script()
def main():

  Script.parseCommandLine()
  args = Script.getPositionalArgs()
  if (len(args) != 1):
    Script.showHelp()

  # get arguments
  transIDs = []
  for arg in args[0].split(','):
    if os.path.exists(arg):
      lines = open(arg, 'rt').readlines()
      for line in lines:
        for transID in line.split(','):
          transIDs += [int(transID.strip())]
    else:
      transIDs.append(int(arg))

  values = []

  for transID in transIDs:
    transName, files_PercentProcessed = get_transformation_info(transID)
    values.append((transName, files_PercentProcessed))

  # print table
  gLogger.notice('\n|_. Transformation Name |_. Files Processed (%)| ')
  for value in values:
    gLogger.notice("|%s|" % "|".join(map(str,value)))

  DIRAC.exit()

if __name__ == '__main__':
  main()



#!/usr/bin/env python
""" Simple data management script for users
    put and register files in DFC or the specified catalog
"""

__RCSID__ = "$Id$"

# generic imports
import os
import glob
import json

# DIRAC imports
import DIRAC
from DIRAC.Core.Utilities.DIRACScript import DIRACScript as Script
Script.parseCommandLine()

# Specific DIRAC imports
from DIRAC.Core.Utilities import List
from CTADIRAC.Core.Workflow.Modules.ProdDataManager import ProdDataManager

####################################################

@Script()
def main():
  """ simple wrapper to put and register all analysis files

  Keyword arguments:
  args -- a list of arguments in order []
  """
  args = Script.getPositionalArgs()
  outputpattern = args[0]
  outputpath = args[1]
  SEListArg = json.loads(args[2])
  SEList = []
  for SE in SEListArg:
    SEList.append(str(SE))
  # Load catalogs
  catalogs = args[3]
  catalogs_json = json.loads(catalogs)

  # # Init DataManager
  prod_dm = ProdDataManager(catalogs_json)

  # # Upload data files
  res = prod_dm._checkemptydir(outputpattern)
  if not res['OK']:
    return res

  for localfile in glob.glob(outputpattern):
    filename = os.path.basename(localfile)
    lfn = os.path.join(outputpath, filename)
    SEList = List.randomize(SEList)
    res = prod_dm._putAndRegisterToSEList(lfn, localfile, SEList)
    # ##  check if failed
    if not res['OK']:
      return res

  DIRAC.exit()


####################################################
if __name__ == '__main__':
  main()


"""
Launcher script to launch Prod5Stage1MergeJob on the WMS or create a Transformation

Usage:
python prod5_stage1_merge_launcher.py TS <trans_prefix> <dataset name or ascii file with a list of datasets> <group_size>

In testing mode (WMS):
python prod5_stage1_merge_launcher.py WMS

Arguments:
mode: WMS for testing, TS for production
Arguments with TS mode:
trans_prefix: prefix to add to the name of the transformation
input_dataset: name of the input dataset
group_size: n files to process

Example:
python prod5_ctapipe_merge_launcher.py TS ctapipe_merge Prod5_Paranal_AdvancedBaseline_NSB1x_gamma-diffuse_North_20deg_DL2_train_en 25
"""
import os
from copy import copy

from DIRAC.Core.Base import Script
Script.parseCommandLine()

import DIRAC
from CTADIRAC.Core.Utilities.tool_box import read_inputs_from_file
from DIRAC.TransformationSystem.Client.Transformation import Transformation
from CTADIRAC.Interfaces.API.Prod5CtaPipeMergeJob import Prod5CtaPipeMergeJob
from DIRAC.Interfaces.API.Dirac import Dirac
from CTADIRAC.Core.Utilities.tool_box import get_dataset_MQ


def submit_trans(job, trans_name, input_meta_query, group_size):
  """ Create a transformation executing the job workflow
  """
  DIRAC.gLogger.notice('submit_trans : %s' % trans_name)

  trans = Transformation()
  trans.setTransformationName(trans_name)  # this must be unique
  trans.setType("Merging")
  trans.setDescription("ctapipe merge")
  trans.setLongDescription("ctapipe merge trans")  # mandatory
  trans.setBody(job.workflow.toXML())
  trans.setGroupSize(group_size)
  trans.setInputMetaQuery(input_meta_query)
  result = trans.addTransformation()  # transformation is created here
  if not result['OK']:
    return result
  trans.setStatus("Active")
  trans.setAgentType("Automatic")
  trans_id = trans.getTransformationID()
  return trans_id


def submit_wms(job):
  """ Submit the job to the WMS
  @todo launch job locally
  """
  dirac = Dirac()
  base_path = '/vo.cta.in2p3.fr/MC/PROD5/Paranal/gamma-diffuse/ctapipe-modeling/3008/Data/000xxx/'
  input_data = ['%s/gamma_20deg_0deg_run100___cta-prod5-paranal_desert-2147m-Paranal-dark_cone10.modeling.DL2.h5' % base_path,
                '%s/gamma_20deg_0deg_run101___cta-prod5-paranal_desert-2147m-Paranal-dark_cone10.modeling.DL2.h5' % base_path]

  job.setInputData(input_data)
  job.setJobGroup('prod5_ctapipe_merge')
  result = dirac.submitJob(job)
  if result['OK']:
    Script.gLogger.notice('Submitted job: ', result['Value'])
  return result

def launch_job(args):
  """ Simple launcher to instanciate a Job and setup parameters
      from positional arguments given on the command line.

      Parameters:
      args -- mode (trans_name dataset_name group_size)
  """
  DIRAC.gLogger.notice('Launching ctapipe merging')
  # get arguments
  mode = args[0]

  # use this mode for testing
  if mode == 'WMS':
    # job setup
    job = Prod5CtaPipeMergeJob(cpuTime=259200.)
    job.version = 'v0.15.0'
    # job.stage1_config = 'stage1_config_Prod3_LaPalma_Baseline_NSB1x.json'
    job.setName('Prod5_ctapipe_merge')
    job.setOutputSandbox(['*Log.txt'])

    job.base_path = '/vo.cta.in2p3.fr/user/a/arrabito'
    output_meta_data = {'array_layout': 'Advanced-Baseline', 'site': 'Paranal',
                     'particle': 'gamma-diffuse', 'phiP': 180.0, 'thetaP': 20.0,
                     'data_level': 2, 'outputType': 'Data',
                     'MCCampaign': 'PROD5', 'stage1_prog': 'ctapipe-merge',
                     'stage1_prog_version': 'v0.15.0'}

    job.set_meta_data(output_meta_data)
    job.set_file_meta_data(nsb=1, split='train_en')
    job.setupWorkflow(debug=True)
    res = submit_wms(job)

  # use this mode for production
  if mode == 'TS':
    # get arguments
    name_prefix = args[1]
    group_size = int(args[3])

    dataset_list = []
    if os.path.isfile(args[2]):
      DIRAC.gLogger.notice('Reading datasets from input file:', args[2])
      dataset_list = read_inputs_from_file(args[2])
    else:
      dataset_list.append(args[2])

    for dataset_name in dataset_list:
      # job setup
      job = Prod5CtaPipeMergeJob(cpuTime=259200.)
      job.version = 'v0.15.0'
      job.setName('Prod5_ctapipe_merge')
      job.setOutputSandbox(['*Log.txt'])
      job.base_path = '/vo.cta.in2p3.fr/MC/PROD5'

      # set meta data for output data

      # start with input metadata
      input_meta_query = get_dataset_MQ(dataset_name)
      output_meta_data = copy(input_meta_query)
      # then refine some metadata in the Job module
      job.set_meta_data(output_meta_data)
      job.set_file_meta_data(nsb=output_meta_data['nsb']['='],split=output_meta_data['split'])

      # define the job steps
      job.setupWorkflow(debug=False)
      job.setType('Merging')  # mandatory *here*

    # submit the transformations
      trans_name = name_prefix + '_' + dataset_name
      res = submit_trans(job, trans_name, input_meta_query, group_size)
  else:
    DIRAC.gLogger.error('1st argument should be the job mode: WMS or TS,\n\
                             not %s' % mode)
    return None

  return res


#########################################################
if __name__ == '__main__':

  arguments = Script.getPositionalArgs()
  if len(arguments) not in [1, 4]:
    Script.showHelp()
  try:
    res = launch_job(arguments)
    if not res['OK']:
      DIRAC.gLogger.error(res['Message'])
      DIRAC.exit(-1)
    else:
      DIRAC.gLogger.notice('Done')
  except Exception:
    DIRAC.gLogger.exception()
    DIRAC.exit(-1)



#!/usr/bin/env python
"""
Launch a production from a workflow description

Usage:
    cta-prod-submit <name of the production> <YAML file with workflow description> <execution mode>
Arguments:
   name of the production
   YAML file with the workflow description
   execution mode is optional:
     ps: submit the workflow to the Production System (default)
     wms: submit individual jobs to the WMS
     local: execute the workflow locally as individual jobs
Example:
    cta-prod-submit TestProd production_config.yml
Example for local execution (for testing):
    cta-prod-submit TestProd production_config.yml local
"""

__RCSID__ = "$Id$"

from DIRAC.Core.Utilities.DIRACScript import DIRACScript as Script

Script.parseCommandLine()
import DIRAC
import yaml
import json

from DIRAC.ProductionSystem.Client.ProductionClient import ProductionClient
from DIRAC.Interfaces.API.Dirac import Dirac
from CTADIRAC.ProductionSystem.Client.SimulationElement import SimulationElement
from CTADIRAC.ProductionSystem.Client.CtapipeModelingElement import (
    CtapipeModelingElement,
)
from CTADIRAC.ProductionSystem.Client.EvnDispElement import EvnDispElement
from CTADIRAC.ProductionSystem.Client.MergingElement import MergingElement


def check_id(workflow_config):
    """Check step ID values"""
    for workflow_step in workflow_config:
        if not workflow_step.get("ID"):
            DIRAC.gLogger.error("Unknown step ID")
            DIRAC.exit(-1)
        elif not isinstance(workflow_step["ID"], int):
            DIRAC.gLogger.error("step ID must be integer")
            DIRAC.exit(-1)
    return True


def sort_by_id(workflow_config):
    """Sort workflow steps by ID"""
    return sorted(workflow_config, key=lambda k: k["ID"])


def check_parents(workflow_config):
    """Check if parent step is listed before child step"""
    for workflow_step in workflow_config:
        if workflow_step["input_meta_query"].get("parentID"):
            if workflow_step["input_meta_query"]["parentID"] > workflow_step["ID"]:
                DIRAC.gLogger.error(
                    "A step can only have a parent which ID is inferior to its ID"
                )
                DIRAC.exit(-1)
    return True


def instantiate_workflow_element_from_type(workflow_step, parent_prod_step):
    """Instantiate workflow element class based on the step type required"""
    wf_elt = None
    if workflow_step["job_config"]["type"].lower() == "mcsimulation":
        wf_elt = SimulationElement(parent_prod_step)
    elif workflow_step["job_config"]["type"].lower() == "ctapipeprocessing":
        wf_elt = CtapipeModelingElement(parent_prod_step)
    elif workflow_step["job_config"]["type"].lower() == "evndispprocessing":
        wf_elt = EvnDispElement(parent_prod_step)
    elif workflow_step["job_config"]["type"].lower() == "merging":
        wf_elt = MergingElement(parent_prod_step)
    else:
        DIRAC.gLogger.error("Unknown step type")
        DIRAC.exit(-1)
    return wf_elt


def find_parent_prod_step(workflow_element_list, workflow_step):
    """Find parent prod step for a given workflow element"""
    parent_prod_step = None
    if workflow_step["input_meta_query"].get("parentID"):
        parent_prod_step = workflow_element_list[
            workflow_step["input_meta_query"]["parentID"] - 1
        ].prod_step  # Python starts indexing at 0
    return parent_prod_step


def get_parents_list(workflow_config):
    parents_list = []
    for workflow_step in workflow_config:
        if workflow_step["input_meta_query"].get("parentID"):
            parents_list.append(workflow_step["input_meta_query"]["parentID"])
    return parents_list


def check_input_source_unicity(workflow_config):
    for workflow_step in workflow_config:
        if workflow_step["input_meta_query"].get("parentID"):
            if workflow_step["input_meta_query"].get("dataset"):
                DIRAC.gLogger.error(
                    "A step cannot have input data from a dataset and from a parent"
                )
                DIRAC.exit(-1)
    return True


def check_destination_catalogs(workflow_element, workflow_step, parents_list):
    """If a change in the destination catalogs is asked by the user, check that the step does not have any children.
    This function needs to be called before setting job attributes."""
    if workflow_step["job_config"].get("catalogs"):
        if workflow_step["ID"] in parents_list:
            # Check that the default catalogs value is different from the catalogs value asked by the user
            # to issue an error
            if workflow_element.job.catalogs != json.dumps(
                workflow_step["job_config"]["catalogs"]
                .replace(", ", ",")
                .split(sep=",")
            ):
                DIRAC.gLogger.error(
                    "Catalogs can only be changed for production steps without any children."
                )
                DIRAC.exit(-1)
    return True


def build_workflow(workflow_config, prod_sys_client, mode):
    """For each workflow step, build the associated workflow element composed of a job and a production step"""
    workflow_element_list = []
    check_id(workflow_config["ProdSteps"])
    workflow_config["ProdSteps"] = sort_by_id(workflow_config["ProdSteps"])
    check_parents(workflow_config["ProdSteps"])
    parents_list = get_parents_list(workflow_config["ProdSteps"])
    for workflow_step in workflow_config["ProdSteps"]:
        parent_prod_step = find_parent_prod_step(workflow_element_list, workflow_step)
        workflow_element = instantiate_workflow_element_from_type(
            workflow_step, parent_prod_step
        )
        check_destination_catalogs(workflow_element, workflow_step, parents_list)
        DIRAC.gLogger.notice(
            "\tBuilding Production step: %s" % workflow_step["job_config"]["type"]
        )
        workflow_element.build_input_data(workflow_config, workflow_step)
        workflow_element.build_job_attributes(workflow_config, workflow_step)
        workflow_element.build_job_input_data(mode)
        workflow_element.build_job_output_data(workflow_step)
        workflow_element.build_element_config()
        workflow_element.build_output_data()
        prod_sys_client.addProductionStep(workflow_element.prod_step)
        workflow_element_list.append(workflow_element)

        # For local and wms mode : build and submit the job in the same loop
        if mode.lower() in ["wms", "local"]:
            res = Dirac().submitJob(workflow_element.job, mode=mode)
            if not res["OK"]:
                DIRAC.gLogger.error(res["Message"])
                DIRAC.exit(-1)
            DIRAC.gLogger.notice("\tSubmitted job:", res["Value"])


@Script()
def main():
    arguments = Script.getPositionalArgs()
    if len(arguments) not in list(range(2, 4)):
        Script.showHelp()

    # Read the arguments
    prod_name = arguments[0]
    workflow_config_file = arguments[1]
    mode = "ps"
    if len(arguments) == 3:
        mode = arguments[2]
    if mode not in ["ps", "wms", "local"]:
        Script.showHelp()

    with open(workflow_config_file, "r") as stream:
        workflow_config = yaml.safe_load(stream)

    ##################################
    # Create the production
    DIRAC.gLogger.notice("Building new production: %s" % prod_name)
    prod_sys_client = ProductionClient()

    ##################################
    # Build production steps according to the workflow description
    build_workflow(workflow_config, prod_sys_client, mode)
    ##################################

    # The default mode is ps, i.e. submit the worflow to the Production System
    if mode.lower() == "ps":
        # Get the production description
        prod_description = prod_sys_client.prodDescription
        # Create the production
        res = prod_sys_client.addProduction(prod_name, json.dumps(prod_description))
        if not res["OK"]:
            DIRAC.gLogger.error(res["Message"])
            DIRAC.exit(-1)

        # Start the production, i.e. instantiate the transformation steps
        res = prod_sys_client.startProduction(prod_name)

        if not res["OK"]:
            DIRAC.gLogger.error(res["Message"])
            DIRAC.exit(-1)

        DIRAC.gLogger.notice("Production %s successfully created" % prod_name)

        # Print the submitted transformations
        res = prod_sys_client.getProductionTransformations(prod_name)
        if res["OK"]:
            transList = res["Value"]
            if not transList:
                DIRAC.gLogger.notice(
                    "No transformation associated with production %s" % prod_name
                )
                DIRAC.exit(-1)
            for trans in transList:
                DIRAC.gLogger.notice(
                    "Submitted transformation:", trans["TransformationID"]
                )
        else:
            DIRAC.gLogger.error(res["Message"])
            DIRAC.exit(-1)


########################################################
if __name__ == "__main__":
    main()



#!/usr/bin/env python

__RCSID__ = "$Id$"

# generic imports
from multiprocessing import Pool

# DIRAC imports
from DIRAC import gLogger
from CTADIRAC.Core.Utilities.tool_box import read_inputs_from_file
from DIRAC.Core.Utilities.DIRACScript import DIRACScript as Script

Script.setUsageMessage("""
Bulk removal of a list of files from the catalog
Usage:
   cta-prod-unregister-file <ascii file with lfn list>
""")

Script.parseCommandLine(ignoreErrors=True)
# Must comes after parseCommandLine
from DIRAC.Resources.Catalog.FileCatalog import FileCatalog
fc = FileCatalog()

def unregisterFile(lfn):
  res = fc.removeFile(lfn)
  if res['OK']:
    if 'Failed' in res['Value']:
      if lfn in res['Value']['Failed']:
        gLogger.error(res['Value']['Failed'][lfn])
      elif lfn in res['Value']['Successful']:
        gLogger.notice("Successfully removed from the catalog", lfn)
      else:
        gLogger.error("Unexpected error result", res['Value'])
    else:
      gLogger.notice("Successfully removed from the catalog", lfn)
  else:
    gLogger.error("Failed to remove file from the catalog:", res['Message'])

  if not res['OK']:
    gLogger.error(res['Message'])

@Script()
def main():
  args = Script.getPositionalArgs()
  if len(args) > 0:
    infile = args[0]
  else:
    Script.showHelp()

  infileList = read_inputs_from_file(infile)
  p = Pool(10)
  p.map(unregisterFile, infileList)

if __name__ == '__main__':
  main()


#!/usr/bin/env python
""" Collection of simple functions to verify each of
    the main Prod3MCJob steps
"""

# generic imports
import os
import glob

# DIRAC imports
import DIRAC
from DIRAC.Core.Utilities.DIRACScript import DIRACScript as Script

Script.setUsageMessage('\n'.join([__doc__.split('\n')[1],
                                  'Usage:',
                                  '  %s stepName' % Script.scriptName,
                                  'Arguments:',
                                  '  stepName: corsika, simtel, merging',
                                  '\ne.g: %s corsika' % Script.scriptName
                                  ]))

Script.parseCommandLine()

def clean_output_file(output_files):
  """ Delete Local Files
  """
  DIRAC.gLogger.notice('Deleting Local Files')
  for afile in output_files:
    DIRAC.gLogger.warn('Remove local File %s' % afile.strip())
    os.remove(afile.strip())

def verify_corsika():
  """ Verify a generic Corsika log file
  """
  DIRAC.gLogger.notice('Verifying Corsika log file')

  # get list of output files
  log_file = glob.glob('Data/corsika/run*/run*.log')
  if len(log_file) != 1:
    DIRAC.gLogger.error('no log file found!')
    DIRAC.exit(-1)

  # check EOR tag
  tag = '=== END OF RUN ==='
  content = open(log_file[0],'rt').read()
  if content.find(tag) < 0:
    DIRAC.gLogger.error('"%s" tag not found!' % tag)
    corsika_files = list(os.popen('find . -iname "*corsika.*z*"'))
    if len(corsika_files) > 0:
      clean_output_file(corsika_files)
    simtel_files = list(os.popen('find . -iname "*simtel.*z*"'))
    if len(simtel_files) > 0:
      clean_output_file(simtel_files)
    log_gz_files = list(os.popen('find . -iname "*log.*z*"'))
    if len(log_gz_files) > 0:
      clean_output_file(log_gz_files)
    log_hist_files = list(os.popen('find . -iname "*log_hist.tar"'))
    if len(log_hist_files) > 0:
      clean_output_file(log_hist_files)
    DIRAC.exit(-1)

def verifySimtel(nbFiles=31, minSize=50.):
  """ Verify a PROD3 simtel step

  Keyword arguments:
  nbFiles -- number of output files expected
  minSize -- minimum file size
  """
  DIRAC.gLogger.notice('Verifying Simtel step')
  # get list of output files
  outputFiles = glob.glob('Data/simtel_tmp/Data/*simtel.*z*')

  # check the number of output files --- could be done by telescope type
  N = len(outputFiles)
  if N != nbFiles:
    DIRAC.gLogger.error('Wrong number of Simtel files : %s instead of %s' %
                        (N, nbFiles))
    clean_output_file(outputFiles)
    DIRAC.exit(-1)

  # check the file size --- could be done by telescope type
  for afile in outputFiles:
    sizekb = os.path.getsize(afile) / 1024.
    if sizekb < minSize:
      DIRAC.gLogger.error('%s\n File size too small : %s < %s kb' % (afile, sizekb, minSize))
      clean_output_file(outputFiles)
      DIRAC.exit(-1)
  DIRAC.gLogger.notice('Good files found:\n%s' % '\n'.join(outputFiles))

def verifyMerging(nbFiles=10, minSize=5000.):
  """ Verify a PROD3 simtel merging step

  Keyword arguments:
  nbFiles -- number of output files expected
  minSize -- minimum file size
  """
  DIRAC.gLogger.notice('Verifying Merging step')

  # get list of output files
  outputFiles = glob.glob('Data/sim_telarray/cta-prod3/0.0deg/Data/*simtel.*z*')

  # check the number of output files --- could be done by telescope type
  N = len(outputFiles)
  if N != nbFiles:
    DIRAC.gLogger.error('Wrong number of Simtel Merged files : %s instead of %s' % (N, nbFiles))
    clean_output_file(outputFiles)
    DIRAC.exit(-1)

  # check the file size --- could be done by telescope type
  for afile in outputFiles:
    sizekb = os.path.getsize(afile) / 1024.
    if sizekb < minSize:
      DIRAC.gLogger.error('%s\n File size too small : %s < %s kb' % (afile, sizekb, minSize))
      clean_output_file(outputFiles)
      DIRAC.exit(-1)
  DIRAC.gLogger.notice('Good files found:\n%s' % '\n'.join(outputFiles))

def verifyAnalysisInputs(minSize=50.):
  """ Verify input files for analysis

  Keyword arguments:
  minSize -- minimum file size
  """
  DIRAC.gLogger.notice('Verifying AnalysisInputs step')

  # get list of output files
  outputFiles = glob.glob('./*simtel.gz')

  # check the file size --- could be done by telescope type
  for afile in outputFiles:
    sizekb = os.path.getsize(afile) / 1024.
    if sizekb < minSize:
      DIRAC.gLogger.warn('%s\n File size too small : %s < %s kb' % (afile, sizekb, minSize))
      DIRAC.gLogger.warn('Remove local File %s' % (afile))
      os.remove(afile)
      outputFiles.remove(afile)  # remove from list of files processed
  DIRAC.gLogger.notice('Good files found:\n%s' % '\n'.join(outputFiles))

def verifyGeneric(nbFiles=1, minSize=50., path='Data/*'):
  """ Verify a PROD3 generic step

  Keyword arguments:
  nbFiles -- number of output files expected
  minSize -- minimum file size
  """
  DIRAC.gLogger.notice('Verifying generic step output')

  # get list of output files
  outputFiles = glob.glob(path)

  # check the number of output files
  N = len(outputFiles)
  if N != nbFiles:
    DIRAC.gLogger.error('Wrong number of output files : %s instead of %s' % (N, nbFiles))
    clean_output_file(outputFiles)
    DIRAC.exit(-1)

  # check the file size
  for afile in outputFiles:
    sizekb = os.path.getsize(afile) / 1024.
    if sizekb < minSize:
      DIRAC.gLogger.error('%s\n File size too small : %s < %s kb' % (afile, sizekb, minSize))
      clean_output_file(outputFiles)
      DIRAC.exit(-1)
  DIRAC.gLogger.notice('Good files found:\n%s' % '\n'.join(outputFiles))

@Script()
def main():
  """ simple wrapper to put and register all PROD3 files

  Keyword arguments:
  args -- a list of arguments in order []
  """
  # check command line
  args = Script.getPositionalArgs()
  if len(args) < 1:
    DIRAC.gLogger.error('requires at least a step type')
    DIRAC.exit(-1)
  elif len(args) == 1:
    stepType = args[0]
  elif len(args) > 3:
    # now do something
    stepType = args[0]
    nbFiles = int(args[1])
    fileSize = float(args[2])
    if len(args) == 4:
      path = args[3]

  # What shall we verify ?
  if stepType == "corsika":
    verify_corsika()
  elif stepType == "simtel":
    verifySimtel(nbFiles, fileSize)
  elif stepType == "merging":
    verifyMerging(nbFiles, fileSize)
  elif stepType == "analysisinputs":
    verifyAnalysisInputs(fileSize)
  elif stepType == "generic":
    verifyGeneric(nbFiles, fileSize, path)
  else:
    DIRAC.gLogger.error('Do not know how to verify "%s"' % stepType)
    DIRAC.exit(-1)
  DIRAC.exit()

if __name__ == '__main__':
  main()


"""
  New job class to run Prod6 single-telescope muon simulations
"""

__RCSID__ = "$Id$"

# generic imports
import json
import collections
import numpy as np
# DIRAC imports
import DIRAC
from DIRAC.Interfaces.API.Job import Job
from CTADIRAC.Core.Utilities.tool_box import DATA_LEVEL_METADATA_ID


class Prod6MCPipeMuonJob(Job):
  """ Job extension class for Prod5 MC NSB simulations,
  takes care of running corsika piped into simtel
  2 output files are created for Dark and Moon NSB
  """

  def __init__(self, cpu_time=259200):
    """ Constructor takes almosst everything from base class

    Keyword arguments:
    cpuTime -- max cpu time allowed for the job
    """
    Job.__init__(self)
    self.setCPUTime(cpu_time)
    self.setName('Prod6MC_Generation')
    self.setType('MCSimulation')
    self.package = 'corsika_simtelarray'
    self.version = '2022-08-03-sc'
    self.compiler = 'gcc83_matchcpu'
    self.program_category = 'tel_sim'
    self.prog_name = 'sim_telarray'
    self.configuration_id = 15
    self.output_data_level = DATA_LEVEL_METADATA_ID['R1']
    self.base_path = '/vo.cta.in2p3.fr/MC/PROD6/'
    self.catalogs = json.dumps(['DIRACFileCatalog', 'TSCatalog'])
    self.metadata = collections.OrderedDict()

  def set_site(self, site):
    """ Set the site to simulate

    Parameters:
    site -- a string for the site name (LaPalma)
    """
    if site in ['Paranal', 'LaPalma']:
      DIRAC.gLogger.info('Set Corsika site to: %s' % site)
      self.cta_site = site
    else:
      DIRAC.gLogger.error('Site is unknown: %s' % site)
      DIRAC.exit(-1)

  def set_telescope(self, telescope):
    """ Set the telescope to simulate

    Parameters:
    telescope -- a string for the telescope name
    """
    if telescope in ['LST', 'MST-FlashCam', 'MST-NectarCam', 'SST', 'SCT']:
      DIRAC.gLogger.info('Set telescope to: %s' % telescope)
      self.telescope = telescope
    else:
      DIRAC.gLogger.error('Telescope %s is unknown, aborting' % telescope)
      DIRAC.exit(-1)

  def set_particle(self, particle):
    """ Set the corsika primary particle

    Parameters:
    particle -- a string for the particle type/name
    """
    if particle in ['muon']:
      DIRAC.gLogger.info('Set Corsika particle to: %s' % particle)
      self.particle = particle
    else:
      DIRAC.gLogger.error('This script can only perform muon simulations, not %s simulations' % particle)
      DIRAC.exit(-1)

  def set_pointing_dir(self, pointing):
    """ Set the pointing direction, North or South

    Parameters:
    pointing -- a string for the pointing direction
    """
    if pointing in ['North', 'South', 'East', 'West']:
      DIRAC.gLogger.info('Set Pointing dir to: %s' % pointing)
      self.pointing_dir = pointing
    else:
      DIRAC.gLogger.error('Unknown pointing direction: %s' % pointing)
      DIRAC.exit(-1)

  def set_half_moon(self, half_moon=False):
    """ Set if to simulate with full-moon conditions

    Parameters:
    half_moon -- a boolean for simulating with half-moon conditions
    """
    if half_moon is True:
      DIRAC.gLogger.info('Set simulations with half-moon conditions')
      self.half_moon = '--with-half-moon'
    else:
      self.half_moon = ''

  def set_degraded_mirror_reflectivity(self, degraded_mirror_reflectivity=False):
    """ Set the values of degraded mirror reflectivity to simulate

    Parameters:
    degraded_mirror_reflectivity -- a boolean to set if to simulate a degraded mirror reflectivity (0.3 to 1.0 in steps of 0.05)
    """
    if degraded_mirror_reflectivity:
      DIRAC.gLogger.info('Set simulations with degraded mirror reflectivities')
      self.degraded_values = np.arange(0.3, 1.05, 0.05)
      self.degraded_mirror = '--degraded-mirror-ref'
    else:
      DIRAC.gLogger.info('Set simulations without degraded mirror reflectivities')
      self.degraded_values = [1]  # No degraded mirror reflectivity
      self.degraded_mirror = ''

  def set_meta_data(self):
    """ define the common meta data of the application
    """
    # The order of the metadata dictionary is important,
    # since it's used to build the directory structure
    self.metadata['array_layout'] = 'Single-telescope'
    self.metadata['site'] = self.cta_site
    self.metadata['particle'] = self.particle
    # for air shower simulation means North=0 and South=180
    # but here piped into tel_sim so North=180 and South=0
    if self.pointing_dir == 'North':
      self.metadata['phiP'] = 180
    if self.pointing_dir == 'South':
      self.metadata['phiP'] = 0
    self.metadata['thetaP'] = float(self.zenith_angle)
    self.metadata[self.program_category + '_prog'] = self.prog_name
    self.metadata[self.program_category + '_prog_version'] = ''.join(self.version.rsplit('-sc', 1))
    self.metadata['data_level'] = self.output_data_level
    self.metadata['configuration_id'] = self.configuration_id
    if self.telescope == 'SCT':
      self.metadata['sct'] = 'True'
    else:
      self.metadata['sct'] = 'False'

  def setupWorkflow(self, debug=False):
    """ Override the base class job workflow to adapt to NSB test simulations
        All parameters shall have been defined before that method is called.
    """
    # step 1 - debug only
    i_step = 1
    if debug:
      ls_step = self.setExecutable('/bin/ls -alhtr', logFile='LS_Init_Log.txt')
      ls_step['Value']['name'] = 'Step%i_LS_Init' % i_step
      ls_step['Value']['descr_short'] = 'list files in working directory'
      i_step += 1

      env_step = self.setExecutable('/bin/env', logFile='Env_Log.txt')
      env_step['Value']['name'] = 'Step%i_Env' % i_step
      env_step['Value']['descr_short'] = 'Dump environment'
      i_step += 1

    # step 2 : use new CVMFS repo
    sw_step = self.setExecutable('cta-prod-setup-software',
                                 arguments='-p %s -v %s -a simulations -g %s -r /cvmfs/sw.cta-observatory.org/software' %
                                 (self.package, self.version, self.compiler),
                                 logFile='SetupSoftware_Log.txt')
    sw_step['Value']['name'] = 'Step%i_SetupSoftware' % i_step
    sw_step['Value']['descr_short'] = 'Setup software'
    i_step += 1

    # step 3 run corsika+sim_telarray
    prod_exe = './dirac_prod6_muon_run'
    prod_args = '--start_run %s --run %s %s %s %s %s %s %s %s' % \
                (self.start_run_number, self.run_number,
                 self.half_moon, self.degraded_mirror, self.telescope,
                 self.cta_site, self.particle, self.pointing_dir,
                 self.zenith_angle)

    cs_step = self.setExecutable(prod_exe, arguments=prod_args,
                                 logFile='CorsikaSimtel_Log.txt')
    cs_step['Value']['name'] = 'Step%i_CorsikaSimtel' % i_step
    cs_step['Value']['descr_short'] = 'Run Corsika piped into simtel'
    i_step += 1

    # step 4 - define meta data, upload file on SE and register in catalogs
    self.set_meta_data()
    md_json = json.dumps(self.metadata)

    meta_data_field = {'array_layout': 'VARCHAR(128)', 'site': 'VARCHAR(128)',
                       'particle': 'VARCHAR(128)',
                       'phiP': 'float', 'thetaP': 'float',
                       self.program_category + '_prog': 'VARCHAR(128)',
                       self.program_category + '_prog_version': 'VARCHAR(128)',
                       'data_level': 'int', 'configuration_id': 'int', 'merged': 'int', 'sct': 'VARCHAR(128)'}
    md_field_json = json.dumps(meta_data_field)
    i_step += 1

    i_substep = 0
    for degraded in self.degraded_values:

      data_output_pattern = 'Data/*dark-ref-degraded-%.2f.simtel.zst' % (degraded)

      # step 5 verify the number of events in the simtel file
      mgv_step = self.setExecutable('dirac_simtel_check',
                                    arguments="'%s'" %
                                    (data_output_pattern),
                                    logFile='Verify_n_showers_Log.txt')
      mgv_step['Value']['name'] = 'Step%s.%s_VerifyNShowers' % (i_step, i_substep)
      mgv_step['Value']['descr_short'] = 'Verify number of showers'

      # Upload and register data - NSB=1 dark
      file_meta_data = {'runNumber': self.run_number, 'nsb': 1}
      file_md_json = json.dumps(file_meta_data)

      dm_step = self.setExecutable('cta-prod-managedata',
                                   arguments="'%s' '%s' '%s' %s '%s' %s %s '%s' Data" %
                                   (md_json, md_field_json, file_md_json,
                                    self.base_path, data_output_pattern, self.package,
                                    self.program_category, self.catalogs),
                                   logFile='DataManagement_dark_ref_degraded_%.2fLog.txt' % (degraded))
      dm_step['Value']['name'] = 'Step%s.%s_DataManagement' % (i_step, i_substep)
      dm_step['Value']['descr_short'] = 'Save data files to SE and register them in DFC'
      i_substep += 1

      # Upload and register log and histo file - NSB=1
      file_meta_data = {}
      file_md_json = json.dumps(file_meta_data)
      log_file_pattern = 'Data/*dark-ref-degraded-%.2f.log_hist.tar' % (degraded)
      log_step = self.setExecutable('cta-prod-managedata',
                                    arguments="'%s' '%s' '%s' %s '%s' %s %s '%s' Log" %
                                    (md_json, md_field_json, file_md_json,
                                     self.base_path, log_file_pattern, self.package,
                                     self.program_category, self.catalogs),
                                    logFile='LogManagement_dark_ref_degraded_%.2fLog.txt' % (degraded))
      log_step['Value']['name'] = 'Step%s.%s_LogManagement' % (i_step, i_substep)
      log_step['Value']['descr_short'] = 'Save log to SE and register them in DFC'
      i_substep += 1

      if self.half_moon == '--with-halfmoon':
        # Now switching to half moon NSB
        # Upload and register data - NSB=5 half moon
        file_meta_data = {'runNumber': self.run_number, 'nsb': 5}
        file_md_json = json.dumps(file_meta_data)
        data_output_pattern = 'Data/*moon-ref-degraded-%.2f.simtel.zst' % (degraded)

        dm_step = self.setExecutable('cta-prod-managedata',
                                     arguments="'%s' '%s' '%s' %s '%s' %s %s '%s' Data" %
                                     (md_json, md_field_json, file_md_json,
                                      self.base_path, data_output_pattern, self.package,
                                      self.program_category, self.catalogs),
                                     logFile='DataManagement_moon_ref_degraded_%.2fLog.txt' % (degraded))
        dm_step['Value']['name'] = 'Step%s.%s_DataManagement' % (i_step, i_substep)
        dm_step['Value']['descr_short'] = 'Save data files to SE and register them in DFC'
        i_substep += 1

        # Upload and register log file - NSB=5
        file_meta_data = {}
        file_md_json = json.dumps(file_meta_data)
        log_file_pattern = 'Data/*moon-ref-degraded-%.2f.log_hist.tar' % (degraded)
        log_step = self.setExecutable('cta-prod-managedata',
                                      arguments="'%s' '%s' '%s' %s '%s' %s %s '%s' Log" %
                                      (md_json, md_field_json, file_md_json,
                                       self.base_path, log_file_pattern, self.package,
                                       self.program_category, self.catalogs),
                                      logFile='LogManagement_moon_ref_degraded_%.2fLog.txt' % (degraded))
        log_step['Value']['name'] = 'Step%s.%s_LogManagement' % (i_step, i_substep)
        log_step['Value']['descr_short'] = 'Save log to SE and register them in DFC'
        i_substep += 1

      i_step += 1

    # Step 6 - debug only
    if debug:
      ls_step = self.setExecutable('/bin/ls -Ralhtr', logFile='LS_End_Log.txt')
      ls_step['Value']['name'] = 'Step%s_LSHOME_End' % i_step
      ls_step['Value']['descr_short'] = 'list files in Home directory'
      i_step += 1

    # Number of showers is passed via an environment variable
    self.setExecutionEnv({
      'NSHOW': '%s' % self.n_shower,
      'CORSIKA_MULTIPLEX_SEQUENTIAL': '1'  # Set CORSIKA multipipe to run in sequential mode (better for single-slot/cpu grid jobs)
    })



"""
  Simple Wrapper on the Job class to run a ctapipe modeling job
"""

__RCSID__ = "$Id$"

# generic imports
import json

# DIRAC imports
from CTADIRAC.Interfaces.API.Prod5Stage1Job import Prod5Stage1Job


class Prod5CtaPipeModelingJob(Prod5Stage1Job):
    """Job extension class for ctapipe stage1 modeling processing"""

    def __init__(self, cpuTime=432000):
        """Constructor

        Keyword arguments:
        cpuTime -- max cpu time allowed for the job
        """
        Prod5Stage1Job.__init__(self)
        self.setCPUTime(cpuTime)
        # defaults
        self.version = "v0.15.0"
        self.setName("ctapipe_modeling")
        self.setType("DL0_Reprocessing")
        self.program_category = "stage1"
        self.prog_name = "ctapipe-modeling"
        self.ctapipe_base_dl2_config = "base_dl2_config.yml"
        self.ctapipe_site_config = "prod5b_paranal_alpha_nectarcam.yml"
        self.output_data_level = 2

    def set_executable_sequence(self, debug=False):
        """Setup job workflow by defining the sequence of all executables
        All parameters shall have been defined before that method is called.
        """
        i_step = 0
        # step 1 -- debug
        if debug:
            ls_step = self.setExecutable("/bin/ls -alhtr", logFile="LS_Init_Log.txt")
            ls_step["Value"]["name"] = "Step%i_LS_Init" % i_step
            ls_step["Value"]["descr_short"] = "list files in working directory"
            i_step += 1

        # step 2
        sw_step = self.setExecutable(
            "cta-prod-setup-software",
            arguments="-p %s -v %s -a stage1 -g %s"
            % (self.package, self.version, self.compiler),
            logFile="SetupSoftware_Log.txt",
        )
        sw_step["Value"]["name"] = "Step%i_SetupSoftware" % i_step
        sw_step["Value"]["descr_short"] = "Setup software"
        i_step += 1

        # step 3 run stage1
        ev_step = self.setExecutable(
            "./dirac_run_ctapipe_modeling",
            arguments=f"--base_config {self.ctapipe_base_dl2_config} \
                                             --site_config {self.ctapipe_site_config} \
                                             --input_ext {self.simtel_ext}",
            logFile="ctapipe_stage1_Log.txt",
        )
        ev_step["Value"]["name"] = "Step%i_ctapipe_stage1" % i_step
        ev_step["Value"]["descr_short"] = "Run ctapipe stage 1"
        i_step += 1

        # step 4 set meta data and register Data
        meta_data_json = json.dumps(self.output_metadata)
        file_meta_data_json = json.dumps(self.output_file_metadata)

        meta_data_field = {
            "array_layout": "VARCHAR(128)",
            "site": "VARCHAR(128)",
            "particle": "VARCHAR(128)",
            "phiP": "float",
            "thetaP": "float",
            "sct": "VARCHAR(128)",
            self.program_category + "_prog": "VARCHAR(128)",
            self.program_category + "_prog_version": "VARCHAR(128)",
            "data_level": "int",
            "configuration_id": "int",
            "merged": "int",
        }
        meta_data_field_json = json.dumps(meta_data_field)

        # register Data
        data_output_pattern = "./Data/*.h5"  # %self.output_data_level
        dm_step = self.setExecutable(
            "cta-prod-managedata",
            arguments="'%s' '%s' '%s' %s '%s' %s %s '%s' Data"
            % (
                meta_data_json,
                meta_data_field_json,
                file_meta_data_json,
                self.base_path,
                data_output_pattern,
                self.package,
                self.program_category,
                self.catalogs,
            ),
            logFile="DataManagement_Log.txt",
        )
        dm_step["Value"]["name"] = "Step%s_DataManagement" % i_step
        dm_step["Value"][
            "descr_short"
        ] = "Save data files to SE and register them in DFC"
        i_step += 1

        # step 5 register Log
        log_file_pattern = "./Data/*.log_and_prov.tgz"
        file_meta_data = {}
        file_meta_data_json = json.dumps(file_meta_data)
        log_step = self.setExecutable(
            "cta-prod-managedata",
            arguments="'%s' '%s' '%s' %s '%s' %s %s '%s' Log"
            % (
                meta_data_json,
                meta_data_field_json,
                file_meta_data_json,
                self.base_path,
                log_file_pattern,
                self.package,
                self.program_category,
                self.catalogs,
            ),
            logFile="LogManagement_Log.txt",
        )
        log_step["Value"]["name"] = "Step%s_LogManagement" % i_step
        log_step["Value"]["descr_short"] = "Save log to SE and register them in DFC"
        i_step += 1

        # step 6 failover step
        failover_step = self.setExecutable(
            "/bin/ls -l", modulesList=["Script", "FailoverRequest"]
        )
        failover_step["Value"]["name"] = "Step%s_Failover" % i_step
        failover_step["Value"]["descr_short"] = "Tag files as unused if job failed"
        i_step += 1

        # Step 7 - debug only
        if debug:
            ls_step = self.setExecutable("/bin/ls -Ralhtr", logFile="LS_End_Log.txt")
            ls_step["Value"]["name"] = "Step%s_LSHOME_End" % i_step
            ls_step["Value"]["descr_short"] = "list files in Home directory"
            i_step += 1



#!/usr/bin/env python
"""
  Create a Transformation for bulk data replication
"""

from DIRAC.Core.Base import Script

Script.setUsageMessage('\n'.join([__doc__.split('\n')[1],
                                  'Usage:',
                                  '  %s [option|cfgfile] ... [File List] ...' % Script.scriptName,
                                  'Arguments:',
                                  '  List of Files to replicate']))

Script.parseCommandLine()

from DIRAC.TransformationSystem.Client.Transformation import Transformation
from DIRAC.TransformationSystem.Client.TransformationClient import TransformationClient
from CTADIRAC.Core.Utilities.tool_box import read_inputs_from_file

def DataReplicationTSExample(args=None):

  if (len(args) != 1):
    Script.gLogger.notice('Wrong number of arguments')
    Script.showHelp()

  infile = args[0]
  infileList = read_inputs_from_file(infile)

  t = Transformation()
  tc = TransformationClient()

  t.setTransformationName("DM_Replication")  # This must be unique
  # t.setTransformationGroup("Group1")
  t.setType("Replication")
  t.setSourceSE(['CYF-STORM-Disk', 'DESY-ZN-Disk'])  # A list of SE where at least 1 SE is the valid one
  t.setTargetSE(['CEA-Disk'])
  t.setDescription("corsika Replication")
  t.setLongDescription("corsika Replication")  # Mandatory

  t.setGroupSize(1)  # Here you specify how many files should be grouped within the same request, e.g. 100

  t.setPlugin("Broadcast")  # Mandatory for replication

  t.addTransformation()  # Transformation is created here
  t.setStatus("Active")
  t.setAgentType("Automatic")

  transID = t.getTransformationID()
  tc.addFilesToTransformation(transID['Value'], infileList)  # Files are added here


if __name__ == '__main__':

  args = Script.getPositionalArgs()

  try:
    DataReplicationTSExample(args)
  except Exception:
    Script.gLogger.exception()



""""

"""

def extension_metadata():
  return {
      "primary_extension": True,
      "priority": 100,
      "setups": {
          "CTA": "dips://dcta-servers03.pic.es:9135/Configuration/Server,"
                  "dips://dcta-agents03.pic.es:9135/Configuration/Server,"
                  "dips://ccdcta-server04.in2p3.fr:9135/Configuration/Server,"
                  "dips://ccdcta-server05.in2p3.fr:9135/Configuration/Server,"
                  "dips://ccdcta-web01.in2p3.fr:9135/Configuration/Server,"
                  "dips://cta-dirac.zeuthen.desy.de:9135/Configuration/Server",
          "CTA-cert": "dips://ccdcta-cert.in2p3.fr:9135/Configuration/Server",
      },
      "default_setup": "CTA",
  }




"""
  Simple Wrapper on the Job class to handle ctapipe stage 1 image analysis
  for the Prod5b simulations
"""

__RCSID__ = "$Id$"

# generic imports
import json
import collections

# DIRAC imports
from DIRAC.Interfaces.API.Job import Job


class Prod5Stage1Job(Job):
    """Job extension class for ctapipe stage1 processing,"""

    def __init__(self, cpuTime=432000):
        """Constructor

        Keyword arguments:
        cpuTime -- max cpu time allowed for the job
        """
        Job.__init__(self)
        self.setCPUTime(cpuTime)
        # defaults
        self.setName("ctapipe_stage1")
        self.package = "ctapipe"
        self.version = "v0.10.0"
        self.compiler = "gcc48_default"
        self.program_category = "stage1"
        self.prog_name = "ctapipe-stage1"
        self.stage1_config = "stage1_config.json"
        self.simtel_ext = "zst"
        self.configuration_id = 7
        self.output_data_level = 1
        self.output_metadata = collections.OrderedDict()
        self.output_file_metadata = collections.OrderedDict()
        self.base_path = "/vo.cta.in2p3.fr/MC/PROD5b/"
        self.catalogs = json.dumps(["DIRACFileCatalog", "TSCatalog"])
        self.group_size = 1

    def set_output_metadata(self, tel_sim_md):
        """Set DL1 meta data

        Parameters:
        tel_sim_md -- metadata dictionary from the telescope simulation
        """
        # # Set directory meta data
        self.output_metadata = collections.OrderedDict()
        self.output_metadata["array_layout"] = tel_sim_md["array_layout"]
        self.output_metadata["site"] = tel_sim_md["site"]
        self.output_metadata["particle"] = tel_sim_md["particle"]
        try:
            phiP = tel_sim_md["phiP"]["="]
        except TypeError:
            phiP = tel_sim_md["phiP"]
        self.output_metadata["phiP"] = phiP
        try:
            thetaP = tel_sim_md["thetaP"]["="]
        except TypeError:
            thetaP = tel_sim_md["thetaP"]
        self.output_metadata["thetaP"] = thetaP
        self.output_metadata["sct"] = tel_sim_md["sct"]
        self.output_metadata[self.program_category + "_prog"] = self.prog_name
        self.output_metadata[self.program_category + "_prog_version"] = self.version
        self.output_metadata["data_level"] = self.output_data_level
        self.output_metadata["configuration_id"] = self.configuration_id
        self.output_metadata["merged"] = 0
        self.output_metadata["MCCampaign"] = self.MCCampaign

    def set_executable_sequence(self, debug=False):
        """Setup job workflow by defining the sequence of all executables
        All parameters shall have been defined before that method is called.
        """
        i_step = 0
        # step 1 -- debug
        if debug:
            ls_step = self.setExecutable("/bin/ls -alhtr", logFile="LS_Init_Log.txt")
            ls_step["Value"]["name"] = "Step%i_LS_Init" % i_step
            ls_step["Value"]["descr_short"] = "list files in working directory"
            i_step += 1

            env_step = self.setExecutable("/bin/env", logFile="Env_Log.txt")
            env_step["Value"]["name"] = "Step%i_Env" % i_step
            env_step["Value"]["descr_short"] = "Dump environment"
            i_step += 1

        # step 2
        sw_step = self.setExecutable(
            "cta-prod-setup-software",
            arguments="-p %s -v %s -a stage1 -g %s"
            % (self.package, self.version, self.compiler),
            logFile="SetupSoftware_Log.txt",
        )
        sw_step["Value"]["name"] = "Step%i_SetupSoftware" % i_step
        sw_step["Value"]["descr_short"] = "Setup software"
        i_step += 1

        # step 3 run stage1
        ev_step = self.setExecutable(
            "./dirac_run_stage1",
            arguments="--config %s --input_ext %s"
            % (self.stage1_config, self.simtel_ext),
            logFile="ctapipe_stage1_Log.txt",
        )
        ev_step["Value"]["name"] = "Step%i_ctapipe_stage1" % i_step
        ev_step["Value"]["descr_short"] = "Run ctapipe stage 1"
        i_step += 1

        # step 4 set meta data and register Data
        meta_data_json = json.dumps(self.output_metadata)
        file_meta_data_json = json.dumps(self.output_file_metadata)

        meta_data_field = {
            "array_layout": "VARCHAR(128)",
            "site": "VARCHAR(128)",
            "particle": "VARCHAR(128)",
            "phiP": "float",
            "thetaP": "float",
            "sct": "VARCHAR(128)",
            self.program_category + "_prog": "VARCHAR(128)",
            self.program_category + "_prog_version": "VARCHAR(128)",
            "data_level": "int",
            "configuration_id": "int",
            "merged": "int",
        }
        meta_data_field_json = json.dumps(meta_data_field)

        # register Data
        data_output_pattern = "./Data/*.h5"  # %self.output_data_level
        dm_step = self.setExecutable(
            "cta-prod-managedata",
            arguments="'%s' '%s' '%s' %s '%s' %s %s '%s' Data"
            % (
                meta_data_json,
                meta_data_field_json,
                file_meta_data_json,
                self.base_path,
                data_output_pattern,
                self.package,
                self.program_category,
                self.catalogs,
            ),
            logFile="DataManagement_Log.txt",
        )
        dm_step["Value"]["name"] = "Step%s_DataManagement" % i_step
        dm_step["Value"][
            "descr_short"
        ] = "Save data files to SE and register them in DFC"
        i_step += 1

        # step 5 register Log
        # log_file_pattern = './*.logs.tgz'
        # file_meta_data = {}
        # file_meta_data_json = json.dumps(file_meta_data)
        # log_step = self.setExecutable('cta-prod-managedata',
        #                               arguments="'%s' '%s' '%s' %s '%s' %s %s '%s' Log" %
        #                               (meta_data_json, meta_data_field_json,
        #                                file_meta_data_json,
        #                                self.base_path, log_file_pattern, self.package,
        #                                self.program_category, self.catalogs),
        #                               logFile='LogManagement_Log.txt')
        # log_step['Value']['name'] = 'Step%s_LogManagement' % i_step
        # log_step['Value']['descr_short'] = 'Save log to SE and register them in DFC'
        # i_step += 1

        # step 6 failover step
        failover_step = self.setExecutable(
            "/bin/ls -l", modulesList=["Script", "FailoverRequest"]
        )
        failover_step["Value"]["name"] = "Step%s_Failover" % i_step
        failover_step["Value"]["descr_short"] = "Tag files as unused if job failed"
        i_step += 1

        # Step 7 - debug only
        if debug:
            ls_step = self.setExecutable("/bin/ls -Ralhtr", logFile="LS_End_Log.txt")
            ls_step["Value"]["name"] = "Step%s_LSHOME_End" % i_step
            ls_step["Value"]["descr_short"] = "list files in Home directory"
            i_step += 1
